#include "HidDInfo.h"
#include <windows.h>
#include <setupapi.h>

typedef struct _HIDD_ATTRIBUTES
{
	ULONG   Size;
	USHORT  VendorID;
	USHORT  ProductID;
	USHORT  VersionNumber;
} HIDD_ATTRIBUTES, *PHIDD_ATTRIBUTES;

typedef VOID    (__stdcall *PHidD_GetHidGuid)(LPGUID);
typedef BOOLEAN (__stdcall *PHidD_GetAttributes)(HANDLE, PHIDD_ATTRIBUTES);
typedef BOOLEAN (__stdcall *PHidD_SetOutputReport)(HANDLE, PVOID, ULONG);

PHidD_GetHidGuid      HidD_GetHidGuid       = NULL;
PHidD_GetAttributes   HidD_GetAttributes    = NULL;
PHidD_SetOutputReport HidD_SetOutputReport  = NULL;

static int initialized = 0;
HINSTANCE hid_lib = NULL;

int init_lib(){
	if (!initialized){
		hid_lib = LoadLibrary(_T("hid.dll"));
		if (!hid_lib){
            return 1;
		}

		HidD_GetHidGuid = (PHidD_GetHidGuid)GetProcAddress(hid_lib, "HidD_GetHidGuid");
		HidD_GetAttributes = (PHidD_GetAttributes)GetProcAddress(hid_lib, "HidD_GetAttributes");
		HidD_SetOutputReport = (PHidD_SetOutputReport)GetProcAddress(hid_lib, "HidD_SetOutputReport");

		if (!HidD_GetHidGuid || !HidD_GetAttributes || !HidD_SetOutputReport){
            return 2;
		}

		initialized = 1;
	}

    return 0; /* OK */
}

TsVHidDInfo HidDEnumerate(int VID, int PID, bool keep_open, bool exclusive, bool overlapped){
    GUID        guid;
    ULONG       len1 = 0, len2 = 0;
    HDEVINFO    hdevinfo;
    SP_INTERFACE_DEVICE_DATA  ifdata;
    PSP_DEVICE_INTERFACE_DETAIL_DATA detail = 0;
	HANDLE      device;
    int i, j;
    int index;
    TsVHidDInfo HidDInfo;
    akar::TsV<HANDLE, 1> Devices;

    if(init_lib()) {
        /* NG */
        return 0;
    }

    HidD_GetHidGuid(&guid);

    hdevinfo = SetupDiGetClassDevs(&guid, 0, 0, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);

    if(hdevinfo == INVALID_HANDLE_VALUE) {		
        return 0;
    }

    for(index=0; ;++index) {
        ifdata.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);

        if(detail){
            free(detail);
            detail = 0;
        }

        if(SetupDiEnumDeviceInterfaces(hdevinfo, NULL, &guid, index, &ifdata) == NULL){			
            break;
        }

        SetupDiGetDeviceInterfaceDetail(hdevinfo, &ifdata, NULL, 0, &len1, NULL);

        detail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(len1);

        len2 = len1;
        detail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

        if(SetupDiGetDeviceInterfaceDetail(hdevinfo, &ifdata, detail, len2, &len1, NULL) == NULL) {			
            break;
        }

        device = CreateFile(detail->DevicePath, exclusive ? 0 : GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, overlapped ? FILE_FLAG_OVERLAPPED : 0, NULL);

        if(device != INVALID_HANDLE_VALUE) {
            HIDD_ATTRIBUTES attrs;

            if(HidD_GetAttributes(device, &attrs)) {
                if((VID == 0 && PID == 0) || (VID == attrs.VendorID && PID == attrs.ProductID)){
                    HidDInfo.add(HidDInfo_t(detail->DevicePath, attrs.VendorID, attrs.ProductID, attrs.VersionNumber, keep_open ? device : 0));
                }
            }

            if(!keep_open) {
                CloseHandle(device);
            }
        }
    }

    if(detail) {
        free(detail);
    }

    return HidDInfo;
}


