#ifndef __HP_COUNTER_H__
#define __HP_COUNTER_H__

#include "akartypes.h"

namespace WiiPC {

struct HighPrecisionCounterT {
    long long       ticks_per_100us;

    long long       ticks_per_1ms;
    long long       ticks_per_2ms;
    long long       ticks_per_3ms;
    long long       ticks_per_4ms;
    long long       ticks_per_5ms;
    long long       ticks_per_10ms;

    void init(void){
        LARGE_INTEGER li;

        QueryPerformanceFrequency(&li);
        ticks_per_100us = li.QuadPart / 1000 / 10;

        ticks_per_1ms   = li.QuadPart / 1000;
        ticks_per_2ms   = li.QuadPart / 500;
        ticks_per_3ms   = li.QuadPart / 333;
        ticks_per_4ms   = li.QuadPart / 250;
        ticks_per_5ms   = li.QuadPart / 200;
        ticks_per_10ms  = li.QuadPart / 100;
    }

    uint32_t GetTimeStamp100us(void){
        LARGE_INTEGER li;
        QueryPerformanceCounter(&li);
        return (uint32_t)(li.QuadPart / ticks_per_100us);
    }

    uint32_t GetTimeStamp5ms(void){
        LARGE_INTEGER li;
        QueryPerformanceCounter(&li);
        return (uint32_t)(li.QuadPart / ticks_per_5ms);
    }

    uint32_t GetTimeStamp1ms(void){
        LARGE_INTEGER li;
        QueryPerformanceCounter(&li);
        return (uint32_t)(li.QuadPart / ticks_per_1ms);
    }

    uint32_t GetTimeStamp2ms(void){
        LARGE_INTEGER li;
        QueryPerformanceCounter(&li);
        return (uint32_t)(li.QuadPart / ticks_per_2ms);
    }

    uint32_t GetTimeStamp3ms(void){
        LARGE_INTEGER li;
        QueryPerformanceCounter(&li);
        return (uint32_t)(li.QuadPart / ticks_per_3ms);
    }

    uint32_t GetTimeStamp4ms(void){
        LARGE_INTEGER li;
        QueryPerformanceCounter(&li);
        return (uint32_t)(li.QuadPart / ticks_per_4ms);
    }

    uint32_t GetTimeStamp10ms(void){
        LARGE_INTEGER li;
        QueryPerformanceCounter(&li);
        return (uint32_t)(li.QuadPart / ticks_per_10ms);
    }

    static HighPrecisionCounterT* instance(void){
        static HighPrecisionCounterT _instance;
        return &_instance;
    }

private:
    HighPrecisionCounterT(){ init();}
};

inline HighPrecisionCounterT* HpCounter(void){
    return HighPrecisionCounterT::instance();
}

uint32_t    GetTimeStamp1ms(void){
    return HpCounter()->GetTimeStamp1ms();
}

uint32_t    GetTimeStamp2ms(void){
    return HpCounter()->GetTimeStamp2ms();
}

uint32_t    GetTimeStamp3ms(void){
    return HpCounter()->GetTimeStamp3ms();
}

uint32_t    GetTimeStamp4ms(void){
    return HpCounter()->GetTimeStamp4ms();
}

uint32_t    GetTimeStamp5ms(void){
    return HpCounter()->GetTimeStamp5ms();
}

uint32_t    GetTimeStamp10ms(void){
    return HpCounter()->GetTimeStamp10ms();
}

} /* namespace WiiPC */
#endif /* __HP_COUNTER_H__ */
