#ifndef __RING_BUFFER_T__
#define __RING_BUFFER_T__

#include "akartypes.h"

template<int N, typename T>
struct ring_buffer_t{
    T               report[N];
    size_t          getter;
    size_t          putter;
    size_t          total;

    ring_buffer_t() : getter(), putter(), total(){
        memset(report, 0, sizeof(report));
    }

    bool put(T& R, T** ptr = 0){
        if(total < N) {
            report[putter] = R;

            if(ptr) {
                *ptr = &(report[putter]);
            }

            total ++;
            putter = (putter + 1) % N;

            return true;
        }

        return false;
    }

    T* put_new(void){
        T* p;

        if(put(T(), &p)){
            return p;
        }

        return 0;
    }

    bool get(T& R){
        if(total) {
            R = report[getter];
            total --;
            getter = (getter + 1) % N;
            return true;
        }

        return false;
    }

    bool get_and_clear(T& R){
        if(total) {
            R = report[getter];
            total = 0;
            getter = putter = 0;
            return true;
        }

        return false;
    }

    bool full(void){
        return total >= N;
    }

    size_t size(void){
        return total;
    }

    void clear(void){
        total  = 0;
        getter = putter = 0;
    }

};

#endif /* __RING_BUFFER_T__ */
