/*
	Akar CC Ltd 2012
	Types of shared pointers
*/
#ifndef __TSANY_HPP__
#define __TSANY_HPP__

#include <string>
//#include <boost/shared_ptr.hpp>
#include <vector>
#include <map>
#include <list>
#include <stdlib.h>

using std::wstring;
using std::string;
////////////////////////////////////////////////////////////////////////////////
//    TsT (type)
////////////////////////////////////////////////////////////////////////////////
namespace DiscIO {
template <typename T>
struct TsT: public std::shared_ptr< T > {
    typedef T Tu;

    TsT(Tu* p): std::shared_ptr<Tu>(p){}
    TsT(): std::shared_ptr<Tu>(new Tu){}

    TsT create(void){
        return TsT(new Tu);
    }
};


typedef TsT<wstring>         TsTwstring;
typedef TsT<std::string>	 TsTstring;

static std::string strwtos(const wstring& s){
	return std::string(s.begin(), s.end());
}

static wstring strstow(const std::string& s){
	return wstring(s.begin(), s.end());
}
////////////////////////////////////////////////////////////////////////////////
//    TsV (vector)
////////////////////////////////////////////////////////////////////////////////
template <typename T, int N>
struct TsV: public std::shared_ptr< std::vector < TsV<T, N-1> > > {
	typedef TsV<T, N-1>                TswType;
	typedef std::vector< TsV<T, N-1> > Tu;

	TsV(Tu* p): std::shared_ptr< Tu > (p) {}
    TsV(): std::shared_ptr<Tu>(new Tu){}

	TswType& operator[] (size_t index) {
		size_t size = get()->size();
		
        if(index >= size) {
            for(size_t i = size; i != index+1; i++) {
                get()->push_back(TswType::create());
            }
        }
        return(*get())[index];
    }

    TswType operator() (size_t index){
        if(get()) {
            size_t size = get()->size();

            if(index >= size) {
                return TswType(0);
            }else{
                return(*get())[index];
            }
        }else{
            return TswType(0);
        }
    }

    size_t size(void) {
        return get()->size();
    }

    void resize(size_t n) {
        get()->resize(n);
    }

    size_t max_element_size(void){
        size_t max_size = 0;

        for(size_t i=0; i<size(); i++) {
            size_t size = operator[](i)->size();
            if(size > max_size) {
                max_size = size;
            }
        }

        return max_size;
    }

    static TsV create(void){
        return TsV<T, N>(new Tu);
    }

    void static_new(void){
        if(get() == 0) {
            swap(TsV<T, N>);
        }
    }

};

template <typename T>
struct TsV<T, 1>: public std::shared_ptr< std::vector<T> > {
    typedef std::vector<T> Tu;

    TsV<T, 1>(Tu* p): std::shared_ptr< Tu > (p) {}
    TsV<T, 1>(): std::shared_ptr< Tu >(new Tu){}

    T& operator [] (size_t index) {
        size_t size = get()->size();

        if(index >= size) {
            for(size_t i = size; i != index+1; i++) {
                get()->push_back(T());
            }
        }
        return(*get())[index];
    }

    bool operator()(size_t index){
        if(get()) {
            size_t size = get()->size();
    
            if(index >= size) {
                return 0;
            }

            return 1;
        }else{
            return 0;
        }
    }

    size_t size(void) {
        if(get()){
            return get()->size();
        }
        return 0;
    }

    void resize(size_t n) {
        get()->resize(n);
    }

    bool is_defined(size_t index){
        return index < size();
    }

    void add(const T& s){
        get()->push_back(s);
    }

    static TsV<T, 1> create(void){
        return TsV<T, 1>(new Tu);
    }

    void static_new(void){
        if(get() == 0) {
            reset(new Tu);
        }
    }

};

// TsVwstring
typedef TsV<wstring, 1>::Tu TuVwstring;
typedef TsV<wstring, 1>     TsVwstring;

// TsV2wstring
typedef TsV<wstring, 2>::Tu TuV2wstring;
typedef TsV<wstring, 2>     TsV2wstring;

// TsV3wstring
typedef TsV<wstring, 3>::Tu TuV3wstring;
typedef TsV<wstring, 3>     TsV3wstring;

// TsVsize_t
typedef TsV<size_t, 1>::Tu TuVsize_t;
typedef TsV<size_t, 1>     TsVsize_t;

// TsVint
typedef TsV<int, 1>::Tu    TuVint;
typedef TsV<int, 1>        TsVint;

// TsVbool
typedef TsV<bool, 1>::Tu   TuVbool;
typedef TsV<bool, 1>::Tu   TsVbool;


static long wstring_to_long_def(const wstring& s, long def_value = 0, int radix = 10){
	wchar_t* p;
	const wchar_t* q = s.c_str();

	long v = wcstol(q, &p, radix);
	
	if(p == q){
		return def_value;
	}

	return v;
}

static double wstring_to_double_def(const wstring& s, double def_value = 0.f){
	wchar_t* p;
	const wchar_t* q = s.c_str();

	double d = wcstod(q, &p);
	
	if(p == q){
		return def_value;
	}

	return d;
}

////////////////////////////////////////////////////////////////////////////////
//    TsM (map)
////////////////////////////////////////////////////////////////////////////////
template <typename T, typename K, int N>
struct TsdM;

template <typename T, typename K>
struct TsConvert {
    K operator()(const T& v);
};

template <>
struct TsConvert<wstring, int>{
    int operator()(const wstring& v){
		return wstring_to_long_def(v);
	}
};

template<>
struct TsConvert<wstring, bool>{
	bool operator()(const wstring& v){
		return wstring_to_long_def(v);
	}
};

template<>
struct TsConvert<wstring, size_t>{
	size_t operator()(const wstring& v){
        return wstring_to_long_def(v);
    }
};

template<>
struct TsConvert<wstring, double>{
	double operator()(const wstring& v){
		return wstring_to_double_def(v);
    }
};

template<>
struct TsConvert<wstring, wstring>{
    wstring operator()(const wstring& v){
        return v;
	}
};

template<>
struct TsConvert<wstring, std::string>{
	std::string operator()(const wstring& v){
    	return std::string(v.begin(), v.end());
	}
};

template<>
struct TsConvert<std::string, wstring>{
	wstring operator()(const std::string& v){
    	return wstring(v.begin(), v.end());
	}
};

template<typename T>
struct TsConvert<T, T>{
	T operator()(const T& v){
		return v;
	}
};

template <typename T, int N>
struct TsM : public std::shared_ptr< std::map< wstring, TsM<T, N-1> > > {
	typedef TsM<T, N-1> Tp;
	typedef std::map<wstring, Tp > Tu;
	typedef typename Tu::iterator           iterator;
	
	TsM(Tu* p): std::shared_ptr< Tu > (p){}
	TsM(): std::shared_ptr< Tu > (new Tu){}

	template<typename K>
	TsdM<T, K, 1>      def(K val)                   {  return TsdM<T, K, 1> (*this, val); }

	TsdM<T, int, N>    def_int(int val)             {  return TsdM<T, int, N> (*this, val);   }
	TsdM<T, bool, N>   def_bool(bool val)           {  return TsdM<T, bool, N>(*this, val);   }
	TsdM<T, double, N> def_double(double val)       {  return TsdM<T, double, N>(*this, val); }
	TsdM<T, wstring, N> def_wstring(const wstring& val){  return TsdM<T, wstring, N>(*this, val); }
	TsdM<T, size_t, N> def_size_t(size_t val)       {  return TsdM<T, size_t, N>(*this, val); }

	size_t size(void){
		if(get() == 0) {
			return 0;
		}

		return get()->size();
	}

	Tp& operator[](const wstring& index){
		if(get()->find(index) == get()->end()) {
            (*get())[index] = Tp();
        }
        return (*get())[index];
    }

    Tp operator()(const wstring& index){
        if(get()) {
            if(get()->find(index) != get()->end()){
                return (*get())[index]; 
            }else{
                return Tp(0);
            }
        }else{
            return Tp(0);
        }
    }

    TsVwstring keys(void){
        TsVwstring R;
        for(Tu::const_iterator i=get()->begin(); i!=get()->end(); i++) {
            R.add(i->first);
        }

        return R;
    }

    void remove(const wstring& key){
        get()->erase(key);
    }

    TsM<T, N> clone(void){
        TsM<T, N> R;

        TsVwstring k = keys();

        for(size_t i=0; k(i); i++) {
            wstring j = k[i];

            R[j] = (*get())[j].clone();
        }

        return R;
    }

    TsM<T, N> shallow_clone(void){
        TsM<T, N> R;

        TsVwstring k = keys();

        for(size_t i=0; k(i); i++) {
            wstring j = k[i];

            R[j] = (*this)[j];
        }

        return R;
    }

    void static_new(void){
        if(get() == 0) {
            swap(TsM<T, N>);
        }
    }
};


template <typename T>
struct TsM<T, 1>: public std::shared_ptr< std::map< wstring, T> > {
    typedef std::map< wstring, T > Tu;
	typedef typename Tu::iterator          iterator;
    typedef T Tr;


	TsM<T, 1>(Tu* p): std::shared_ptr< Tu >(p) {}
	TsM<T, 1>(): std::shared_ptr< Tu >(new Tu) {}

	template<typename K>
	TsdM<T, K, 1>      def(K val)                   {  return TsdM<T, K, 1> (*this, val); }

	TsdM<T, int, 1>    def_int(int val)             {  return TsdM<T, int, 1> (*this, val);   }
	TsdM<T, bool, 1>   def_bool(bool val)           {  return TsdM<T, bool, 1>(*this, val);   }
	TsdM<T, double, 1> def_double(double val)       {  return TsdM<T, double, 1>(*this, val); }
	TsdM<T, wstring, 1> def_wstring(const wstring& val){  return TsdM<T, wstring, 1>(*this, val); }
	TsdM<T, size_t, 1> def_size_t(size_t val)       {  return TsdM<T, size_t, 1>(*this, val); }

	T& operator[](const wstring& index){
		if(get()->find(index) == get()->end()) {
			(*get())[index] = T();
		}

        return (*get())[index];
    }

	static bool exists(const Tu* X, const wstring& index){
		return X->find(index) != X->end();
	}

	bool operator()(const wstring& index){
		if(get()) {
            return exists(get(), index);
        }else{
            return 0;
        }
    }

    size_t size(void){
        if(get() == 0) {
            return 0;
        }

        return get()->size();
    }

    TsVwstring keys(void){
        TsVwstring R;
        for(Tu::const_iterator i=get()->begin(); i!=get()->end(); i++) {
            R.add(i->first);
        }

        return R;
    }

    void remove(const wstring& key){
        get()->erase(key);
    }

    TsM<T, 1> clone(void){
        return TsM<T, 1>(new Tu(*get()));
    }

    void static_new(void){
        if(get() == 0) {
            swap(TsM<T, 1>());
		}
    }
};


template <typename T, int N>
size_t MaxKeyInteger(TsM<T, N> src){
    size_t n = 0;

    for(TsM<T, N>::Tu::iterator I=src->begin(); I!=src->end(); ++I) {
        size_t m = I->first.ToIntDef(0);
        if(m > n) {
            n = m;
        }
    }

    return n;
}

template <typename T, typename K, int N>
struct TsdM: public TsM<T, N> {
    typedef TsdM<T, K, N-1> Tp;
    K defVal;

    TsdM(TsM<T, N>& Ref, const K& Val): TsM<T, N>(Ref), defVal(Val){}

    Tp operator[](const wstring& index){
        return Tp((*get())[index], defVal);
    }

    TsM<T, N> no_default(void){
        return *this;
    }
};


template <typename T, typename K>
struct TsdM<T, K, 1>: public TsM<T, 1> {
    K defVal;

    TsdM(TsM<T, 1>& Ref, const K& Val): TsM<T, 1>(Ref), defVal(Val){}

    K operator[](const wstring& index){
        if(TsM<T, 1>::operator()(index)){
            return TsConvert<T, K>()
             (TsM<T, 1>::operator[](index));
        }

        return defVal;
    }

};

typedef TsM<wstring, 1>  TsMwstring;
typedef TsM<wstring, 2>  TsM2wstring;
typedef TsM<wstring, 3>  TsM3wstring;
typedef TsM<size_t, 1>  TsMsize_t;
typedef TsM<int, 1>     TsMint;
typedef TsM<bool, 1>    TsMbool;
////////////////////////////////////////////////////////////////////////////////
// TsL
////////////////////////////////////////////////////////////////////////////////
template <typename T, int N>
struct TsL: public std::shared_ptr< std::list< TsL< T, N-1> > > {
    typedef TsL< T, N-1>              Tp;
    typedef std::list< TsL< T, N-1> > Tu;
    
    TsL(Tu* p): std::shared_ptr< Tu > (p){}
    TsL()     : std::shared_ptr< Tu > (new Tu){}

    typename Tu::iterator front_iterator(size_t index){
        Tu::iterator P = get()->begin();

        for(size_t i=0; i<index; i++) {
            P ++;
        }

        return P;
    }

    typename Tu::iterator back_iterator(size_t index, bool advance){
        Tu::iterator P = get()->begin();

        for(size_t i=1; i< (size() - index); i++){
            P ++;
        }

        if(advance) {
            P ++;
        }

        return P;
    }

    typename Tu::reverse_iterator back_iterator2(size_t index){
        Tu::reverse_iterator P = get()->rbegin();

        for(size_t i=0; i<index; i++) {
            P++;
        }

        return P;
    }

    Tp& front(size_t index){
        if(index >= size()) {
            get()->resize(index+1);
        }

        return *front_iterator(index);
    }

    Tp& back(size_t index){
        if(index >= size()) {
            get()->resize(index+1);
        }

        return *back_iterator2(index);
    }

    Tp& insert_front(size_t index){
        if(index > size()) {
            get()->resize(index);
        }

        Tu::iterator P = front_iterator(index);
        P = get()->insert(P, Tp());

        return *P;
    }

    Tp& insert_back(size_t index){
        if(index > size()) {
            get()->resize(index);
        }

        Tu::iterator P = back_iterator(index, true);

        P = get()->insert(P, Tp());

        return *P;
    }

    Tp erase_front(size_t index){
        if(index >= size()) {
            return Tp();
        }

        Tu::iterator P = front_iterator(index);
        Tp S = *P;

        get()->erase(P);

        return S;
    }

    Tp erase_back(size_t index){
        if(index >= size()) {
            return Tp();
        }

        Tu::iterator P = back_iterator(index, false);
        Tp S = *P;

        get()->erase(P);

        return S;
    }


    Tp& operator[](int index){
        if(index >= 0){
            return front(index);
        }else{
            return back(-index-1);
        }
    }

    bool   operator()(int index){
        if(index < 0){
            index = -index -1;
        }

        return index < size();
    }

    size_t size(void){
        return get()->size();
    }

    Tp& push(int index){
        if(index >=0) {
            return insert_front((size_t)index);
        }else{
            return insert_back((size_t)(-index-1));
        }
    }

    Tp pop(int index){
        if(index >=0) {
            return erase_front((size_t)index);
        }else{
            return erase_back((size_t)(-index-1));
        }
    }

    void static_new(void){
        if(get() == 0) {
            reset(new Tu);
        }
    }
};

template <typename T>
struct TsL<T, 1>: public std::shared_ptr< std::list< T > > {
    typedef std::list< T > Tu;

    TsL(Tu* p): std::shared_ptr< std::list<T> > (p){}
    TsL()     : std::shared_ptr< std::list<T> > (get() == 0 ? new Tu : get()){}


    typename Tu::iterator front_iterator(size_t index){
        Tu::iterator P = get()->begin();

        for(size_t i=0; i<index; i++) {
            P ++;
        }

        return P;
    }

    typename Tu::iterator back_iterator(size_t index, bool advance){
        Tu::iterator P = get()->begin();

        for(size_t i=1; i< (size() - index); i++){
            P ++;
        }

        if(advance)
            P++;


        return P;
    }

    typename Tu::reverse_iterator back_iterator2(size_t index){
        Tu::reverse_iterator P = get()->rbegin();

        for(size_t i=0; i<index; i++) {
            P++;
        }

        return P;
    }

    T& front(size_t index){
        if(index >= size()) {
            get()->resize(index+1);
        }

        return *front_iterator(index);
    }

    T& back(size_t index){
        if(index >= size()) {
            get()->resize(index+1);
        }

        return *back_iterator2(index);
    }

    T& insert_front(size_t index, const T& val = L""){
        if(index > size()) {
            get()->resize(index);
        }

        Tu::iterator P = front_iterator(index);
        P = get()->insert(P, val);

        return *P;
    }

    T& insert_back(size_t index, const T& val = T()){
        if(index > size()) {
            get()->resize(index);
        }

        Tu::iterator P = back_iterator(index, true);

        P = get()->insert(P, val);

        return *P;
    }

    T erase_front(size_t index){
        if(index >= size()) {
            return T();
        }

        Tu::iterator P = front_iterator(index);
        T S = *P;

        get()->erase(P);

        return S;
    }

    T erase_back(size_t index){
        if(index >= size()) {
            return T();
        }

        Tu::iterator P = back_iterator(index, false);
        T S = *P;

        get()->erase(P);

        return S;
    }
    

    T& operator[](int index){
        if(get() == 0) {
            reset(new Tu);
        }

        if(index >= 0){
            return front(index);
        }else{
            return back(-index-1);
        }
    }

    bool   operator()(int index){
        if(index < 0){
            index = -index -1;
        }

        return index < size();
    }

    size_t size(void){
        return get()->size();
    }

    T& push(int index, const T& val = T()){
        if(index >=0) {
            return insert_front((size_t)index, val);
        }else{
            return insert_back((size_t)(-index-1), val);
        }
    }

    T pop(int index){
        if(index >=0) {
            return erase_front((size_t)index);
        }else{
            return erase_back((size_t)(-index-1));
        }
    }

    void static_new(void){
        if(get() == 0) {
            reset(new Tu);
        }
    }
};

////////////////////////////////////////////////////////////////////////////////
// TsB
// Shared Buffer
////////////////////////////////////////////////////////////////////////////////
template <typename T>
struct TsB: public std::shared_ptr< T > {
    typedef T Tu;
    size_t  size_;

    TsB(Tu* p, size_t _size): std::shared_ptr<Tu>(p), size_(_size){
    }

    TsB(size_t _size = 0, bool fill = 0, Tu def_value = 0): std::shared_ptr<Tu>(new Tu[_size]), size_(_size){
        if(fill) {
            Tu* p = get();

            for(size_t i=0; i<size_; ++i) {
                *p++ = def_value;
            }
        }
    }

    TsB(TsV<Tu, 1>& svec)
        :std::shared_ptr<Tu>(new Tu[svec.size()]), size_(svec.size()){

        Tu* p = get();

        for(size_t i=0; i<size_; ++i){
            p[i] = svec[i];
        }
    }

    TsB(const TsB<Tu>& x)
        :std::shared_ptr<Tu>(new Tu[x.size_]), size_(x.size_){

        Tu* p = x.get();
        Tu* q = get();
        for(size_t i=0; i<size_; ++i){
            q[i] = p[i];
        }
    }

    TsB<Tu> range(size_t first, size_t last, Tu def_value = 0){
        size_t n = last - first;

        TsB<Tu> R(n);

        Tu* p = R.get();
        Tu* q = get();

        for(size_t i=first, j=0; i!=last; ++i, ++j) {
            if(i<size_) {
                p[j] = q[i];
            }else{
                p[j] = def_value;
            }
        }

        return R;
    }

    TsB<Tu>& assign(size_t pos, TsB<Tu>& src, bool extendable = false, Tu def_value = 0){
        if(extendable) {
            size_t new_size = pos + src.size;

            if(new_size > size) {
                resize(new_size, def_value);
            }
        }

        Tu* p = get();
        Tu* q = src.get();

        for(size_t i=0, j=pos; i<src.size_; ++i, ++j) {
            if(j < size_) {
                p[j] = q[i];
            }else{
                break;
            }
        }

        return *this;
    }

    Tu& operator[](size_t index){
        return get()[index];
    }

    bool operator()(size_t index){
        return index >= size_;
    }

    Tu* operator&(void){
        return get();
    }

    TsB<Tu>& resize(size_t new_size, Tu def_value = 0){
        TsB<Tu> k(range(0, new_size, def_value));
        swap(k);
        return *this;
    }

    TsB<Tu>& extend(size_t more, Tu def_value = 0){
        return resize(size_ + more, def_value);
    }

    void assign_to(Tu* dest, size_t first, size_t last, Tu def_value = 0){
        Tu* p = get();

        for(size_t i = first; i != last; ++i) {
            if(i < size_) {
                *dest = p[i];
            }else{
                *dest = def_value;
            }

            dest ++;
        }
    }

    size_t size(void){
        return size_;
    }

    void static_new(void){
        if(get() == 0) {
            reset(new Tu);
        }
    }
};

template<class Tu>
TsV<Tu, 1>  produce(TsB<Tu>& src){
    TsV<Tu, 1> v;

    for(size_t i=0; i<src.size(); ++i){
        v[i] = src[i];
    }

    return v;
}

typedef TsB<unsigned char>  TsBu8;
typedef TsB<unsigned short> TsBu16;

typedef TsL<wstring, 1>      TsLwstring;
typedef TsL<wstring, 2>      TsL2wstring;

/* Misc */
typedef TsL< TsMwstring, 1 > TsLMwstring;
typedef TsV< TsM2wstring, 1> TsVM2wstring;


/*
	StaticTsM wraps for static objects whose
	member functions may be called before
	the constructor is called.
	StaticTsM shall be only used in
	global scope or static function variable scope.
*/
template<typename T>
struct StaticTsM {
	typedef TsM<T, 1>	Tm;
	typedef typename Tm::Tu		Tu;

	Tu* ptr;

	StaticTsM() /* no init member */ {}
	~StaticTsM(){
		if(ptr) delete ptr;
	}

	void static_new(void){
		if(ptr == 0){
        	ptr = new Tu;
		}
	}

	bool operator()(const wstring& index){
    	return Tm::exists(ptr, index);
	}

	T& operator[](const wstring& index){
        return (*ptr)[index];
	}
};

}

#endif /* __TSANY_HPP__ */

