#ifndef __WIIPCCOMMON_H__
#define __WIIPCCOMMON_H__

#include "akartypes.h"
#include "udata.h"
#include "semantics.h"
#include "WiiPCErr.h"
#include "StdThread.h"
#include "StdMutex.h"
#include "umajor.h"
#include "ucore_minor.h"

#include "loader_minor.h"

#include "manager_minor.h"
#include "hidhost_minor.h"
#include "../../Core/Src/HW/Wiimote.h"

//extern unsigned int g_wiimote_sources[4];

namespace WiiPC {
    struct ManagerT;
    struct WriterT;
    struct ReaderT;
    struct MapperT;
    struct EventerT;
    struct PresenterT;
    

    typedef void (*event_callback_t)(uint16_t m, void* p);
    typedef uint32_t event_id_t;

    const   uint16_t VID = 0x272D;
    const   uint16_t PID = 0x0100;
    const   uint32_t default_read_timeout = 30;
    const   uint16_t usb_payload_size = 0x60;

    ManagerT*   Manager(void);
    WriterT*    Writer(void);
    ReaderT*    Reader(void);
    MapperT*    Mapper(void);
    PresenterT* Presenter(void);
    EventerT*   Eventer(void);

    void  ReaderStart(void);
    void  ReaderStop(void);
    void  ReaderReset(int __core index);
    void  ReaderResetAll(void);
    int   ReaderTryRead(DWORD timeout, udata_t* _udata = 0);

    void  WriterStart(void);
    void  WriterStop(void);
    void  WriterReset(int __core index);
    void  WriterResetAll(void);
    int   WriterWrite(udata_t* udata);

    bool  ManagerIsBridged(void);
    void  ManagerNotify(udata_t* d, uevent_t* e);
    void  ManagerUsbDisconnect(void);
    void  ManagerUsbConnect(void);
    bool  ManagerUsbTryConnect(void);
    bool  ManagerIsUsbConnected(void);
    bool  ManagerIsUsbDisconnected(void);
    void* ManagerUsbHandle(void);
    void  ManagerStart(void);

    void  PresenterStart(void);
    void  PresenterStop(void);

    #if 0
    void  MapperInit    (unsigned int _g_wiimote_sources[4], uint8_t* _conn_bitmap);
    void  MapperReset   (void);
    void  MapperRemap   (void);
    void  MapperDump    (void);
    #endif

    void  EventerStart  (void);
    event_id_t  EventerSet   (event_callback_t callback, uint16_t m, void* p, uint32_t delay_ms, uint32_t group = 0);
    bool  __ok  EventerCancel(event_id_t event_id, uint16_t* pm, void** pp);
    uint32_t  __num EventerCancelGroup(uint32_t group, event_callback_t cancel_callback = 0);

    bool __valid MapperWiiPCToCore(int __wiipc index, int* __core  mapped_index);
    bool __valid MapperCoreToWiiPC(int __core  index, int* __wiipc mapped_index);
    bool __valid ManagerIsWmConnected(int __wiipc index);

    int __from_one_but_zero __mapped  MapperNotifyConnected     (int index);
    int __from_one_but_zero __mapped  MapperNotifyDisconnected  (int index);


    /* common variables */
    extern uint16_t    current_connection;
    extern uint8_t     bridge_mode;

    /* Transx */
    extern std::mutex            mutex_transx_vars;
    extern std::recursive_mutex  mutex_transx;
    extern uint8_t               transx_major;
    extern uint8_t               transx_minor;
    extern bool                  transx_waiting;
    extern bool                  transx_received;
    extern udata_t               transx_udata;

    uparam_t* SetupUevent       (uevent_t* e, uint8_t major, uint8_t minor, uint16_t num_params, uint8_t len_tail);
    uparam_t* SetupUdataUevent  (udata_t* d, uint8_t major, uint8_t minor, uint8_t nparams, uint8_t nb_tail);
    uint16_t  UeventType        (uevent_t* uevent);


    /* Interfaces */
    bool query_status        (uint8_t* bridge_mode, 
                              uint8_t* connection_bitmap, 
                              uint8_t* ir_status, 
                              uint8_t* ir_pos, 
                              uint8_t* discovering);

    bool enable_bridge       (void);
    bool disable_bridge      (void);
    bool send_present        (void);
    bool send_absent         (void);

    bool wiimote_get_states         (uint8_t states[4]);
    bool wiimote_disconnect         (uint8_t device_index);
    bool wiimote_set_leds           (uint8_t index, uint8_t leds, uint8_t ack);
    bool wiimote_set_channel        (uint8_t index, uint8_t channel, uint8_t continuous, uint8_t ack);
    bool wiimote_enable_ir_clock    (uint8_t index, uint8_t enabled, uint8_t ack);
    bool wiimote_enable_audio_clock (uint8_t index, uint8_t enabled, uint8_t ack);
    bool wiimote_read_status        (uint8_t index);
    bool wiimote_write_device       (uint8_t index, uint8_t device, uint8_t mode, uint16_t offset, const uint8_t* src, uint8_t size);
    bool wiimote_read_device        (uint8_t index, uint8_t device, uint8_t mode, uint16_t offset, uint16_t size);
    bool wiimote_write_audio        (uint8_t index, const uint8_t* src, uint8_t size);
    bool wiimote_mute_audio         (uint8_t index, uint8_t mute, uint8_t ack);
    bool wiimote_enable_ir          (uint8_t index, uint8_t enabled, uint8_t ack);

	uint8_t get_command				(uint8_t index, uint8_t* command);

    int  connected_num(void);
    int  __wiipc connected_first_index(void);


    /* Interface to Core/Wiimote */
    void Initialize (void);
    void Shutdown   (void);
    void Update     (int index);

    void InterruptChannel(int number, uint16_t channel, const void* buffer, uint32_t size);
    void ControlChannel  (int number, uint16_t channel, const void* buffer, uint32_t size);
}
/* namespace WiiPC */

#endif /* __WIIPCCOMMON_H__ */
