#include "WiiPCEventer.h"
#include "dconsole.h"

namespace WiiPC{

EventerT::EventerT()
    :ThreadBaseT(false /* high_prio */), n_cell(), mutex_cell()
{
    memset(cell, 0, sizeof(cell));
    max_id = 1;
}

void EventerT::func(void) {
    while(started) {
        mutex_cell.lock();
        for(size_t i=0; i<n_cell; ++i) {
            if(cell[i].flags) {
                if(cell[i].delay > 100) {
                    cell[i].delay -= 100;
                }else{
                    cell[i].delay = 0;
                }

                if(cell[i].delay == 0) {
                    /* let it happen */
                    (cell[i].callback)(cell[i].m, cell[i].p);

                    cell[i].flags = 0;

                    cal_n_cell();
                }
            }
        }
        mutex_cell.unlock();

        Sleep(100);
    }
}

void EventerT::cal_n_cell(void){
    uint32_t m = 0;

    for(size_t i=0; i<EVENTER_CELL_SIZE; ++i) {
        if(cell[i].flags) {
            m = i+1;
        }
    }

    n_cell = m;
}

event_id_t EventerT::set_event   (event_callback_t callback, uint16_t m, void* p, uint32_t delay_ms, uint32_t group){
    std::lock_guard<std::recursive_mutex> lk(mutex_cell);

    if(n_cell >= EVENTER_CELL_SIZE) {
        /* full */
        return 0;
    }

    max_id ++;

    if(max_id == 0) {
        max_id++;
    }

    size_t j = 0;
    bool   valid = false;

    for(size_t i=0; i<n_cell; ++i) {
        if(cell[i].flags == 0) {
            j = i;
            valid = true;
        }
    }

    if(!valid) {
        j = n_cell;
        n_cell ++;
    }

    cell[j].id         = max_id;
    cell[j].callback   = callback;
    cell[j].delay      = delay_ms;
    cell[j].m          = m;
    cell[j].p          = p;
    cell[j].group      = group;
    cell[j].flags      = 1;

    cal_n_cell();

	return 0;
}

bool __ok   EventerT::cancel_event(event_id_t event_id, uint16_t* pm, void** pp){
    std::lock_guard<std::recursive_mutex> lk(mutex_cell);

    for(size_t i=0; i<n_cell; ++i) {
        if(cell[i].id == event_id) {
            if(pm) {
                *pm = cell[i].m;
            }

            if(pp) {
                *pp = cell[i].p;
            }

            cell[i].flags = 0;

            cal_n_cell();

            return true;
        }
    }

	return false;
}

uint32_t __num  EventerT::cancel_group(uint32_t group, event_callback_t cancel_callback){
    std::lock_guard<std::recursive_mutex> lk(mutex_cell);
    uint32_t n = 0;
    uint32_t m = 0;

    for(size_t i=0; i<n_cell; ++i) {
        if(cell[i].group == group){
            if(cancel_callback) {
                cancel_callback(cell[i].m, cell[i].p);
            }

            cell[i].flags = 0;
            n ++;
            m = i+1;
        }
    }

    cal_n_cell();
    return n;
}


}   /* namespace WiiPC */


