#include "WiiPCManager.h"
#include "dconsole.h"

namespace WiiPC {

bool ManagerT::update_status(void){
    return query_status(&status.bridge_mode,
                        &status.conn_bitmap,
                        &status.ir_status,
                        &status.ir_pos,
                        &status.discovering);
}

bool ManagerT::wait_recv_bt_rx(int index, uint8_t channel, uint32_t timeout){
    udata_t udata;

    return recv_bt_rx(&udata, index, channel, timeout) == ERR_OK;
}

bool ManagerT::set_wiimote_to_basic(int __wiipc index, bool wait_ack){
    if(!wiimote_set_channel(index, 0x30, 0, 0)) return false;
    if(wait_ack) {
        if(!wait_recv_bt_rx(index, 0x22, 1000))     return false;
    }
    return true;
}

bool ManagerT::set_all_wiimotes_to_basic(void){
    for(int i=0; i<4; ++i){
        if(status.conn_bitmap & (1<<i)){
            set_wiimote_to_basic(i);
        }
    }
    return true;
}

bool ManagerT::play_leds1(void){
    update_status();

    int j;

    for(int i=0; i<4; ++i) {
        if(status.conn_bitmap & (1<<i)) {
            if(MapperWiiPCToCore(i, &j)) {
                if(!wiimote_set_leds(i, 1, 0)) return false;                
            }
        }
    }
    Sleep(100);

    for(int i=0; i<4; ++i) {
        if(status.conn_bitmap & (1<<i)) {
            if(MapperWiiPCToCore(i, &j)) {
                if(!wiimote_set_leds(i, 2, 0)) return false;
            }
        }
    }
    Sleep(100);

    for(int i=0; i<4; ++i) {
        if(status.conn_bitmap & (1<<i)) {
            if(MapperWiiPCToCore(i, &j)) {
                if(!wiimote_set_leds(i, 4, 0)) return false;
            }
        }
    }
    Sleep(100);

    for(int i=0; i<4; ++i) {
        if(status.conn_bitmap & (1<<i)) {
            if(MapperWiiPCToCore(i, &j)) {
                if(!wiimote_set_leds(i, 8, 0)) return false;
            }
        }
    }
    Sleep(100);

    for(int i=0; i<4; ++i) {
        if(status.conn_bitmap & (1<<i)) {
            if(MapperWiiPCToCore(i, &j)) {
                if(!wiimote_set_leds(i, (1<<j), 0)) return false;
            }
        }
    }

    return true;
}

bool ManagerT::play_leds2(int index){
    int j;

    if(index >= 4) {
        return false;
    }

    if(!MapperWiiPCToCore(index, &j)) {
        return false;
    }

    if(!wiimote_set_leds(index, 1+2, 0)) return false;
    Sleep(200);
    if(!wiimote_set_leds(index, 2+4, 0)) return false;
    Sleep(200);
    if(!wiimote_set_leds(index, 4+8, 0)) return false;
    Sleep(200);
    if(!wiimote_set_leds(index, 8+1, 0)) return false;
    Sleep(200);

    if(!wiimote_set_leds(index, (1<<j), 0)) return false;

    return true;
}

void ManagerT::detach_core_wiimote(int index){
    //dprintf("detach(%d) ", index);

    if(Wiimote::IsCoreInited()) {
        //dprintf("ok ");			
        detach_flags |= (1<<index);		
        Wiimote::Activiate(index, 0);

        int i;
        for(i=0; i<300; ++i) {
            if(detach_flags & (1<<index)) {
                Sleep(10);
            }else{
                //dprintf("ack ");
                break;
            }
        }
    }
}

void ManagerT::attach_core_wiimote(int index){
    dprintf("attach(%d) ", index);

    if(Wiimote::IsCoreInited()) {
		//Wiimote::Activiate(index, 0);
        //dprintf("ok ");
        Wiimote::Activiate(index, 1);
    }
}

void ManagerT::detach_core_wiimotes(void){
    for(int i=0; i<4; ++i) {
        if((g_wiimote_sources[i] & 2) /* WIIMOTE_SRC_REAL */ || ((g_wiimote_sources[i] & 4) && i == 1) /* WIIMOTE_SRC_WII_BALANCE_BOARD */) {
            detach_core_wiimote(i);
        }
    }
}

void ManagerT::attach_core_wiimotes(void){
    for(int j=0; j<4; ++j) {
        if(core_to_wiipc(j, 0)){
            attach_core_wiimote(j);
        }
    }
}

void ManagerT::pause_game(void){
    //dprintf("Pause.Game ");
    Wiimote::CorePauseGame();
}

void ManagerT::resume_game(void){
    //dprintf("Resume.Game ");
    Wiimote::CoreStartGame();
}

void ManagerT::warn_bridged_no_slot(void){
    //dprintf("WARN: Bridged but no slot\n");
}

bool ManagerT::is_usb_connected(void){
    return usb_handle != 0;
}

bool ManagerT::is_bridged(void){
    return state == Bridged;
}

bool ManagerT::is_usb_disconnected(void){
    return state == Disconnected;
}

void ManagerT::usb_disconnect(void){
    if(usb_handle) {
        CloseHandle(usb_handle);
        usb_handle = 0;
        SetEvent(event_disconnected);
    }
}

void ManagerT::usb_connect(void){
    ReaderStop();
    WriterStop();
    usb_disconnect();

    TsVHidDInfo V = HidDEnumerate(VID, PID,
                                  true /* keep_open */,
                                  false /* exclusive */,
                                  true /* overlapped */);
	
    for(size_t i=0; i<V.size(); ++i){
        if(V[i].path == usb_dev_path){
            /* found previous one */
            usb_handle = V[i].handle;
            V[i].handle = 0;
            break;
        }
    }

    if(usb_handle == 0 && V.size()){
        /* no previous one, so we pick the first one */
        usb_handle = V[0].handle;
        usb_dev_path = V[0].path;
        V[0].handle = 0;
    }

    if(usb_handle == 0){
        /* no valid device, also invalidate its path */
        usb_dev_path = "";
    }

    for(size_t i=0; i<V.size(); ++i){
        void* h = V[i].handle;

        if(h){
            CloseHandle(h);
        }
    }

    if(is_usb_connected()) {
        ReaderStart();
        WriterStart();
    }
}

bool ManagerT::usb_try_connect(void){
    if(mutex_try_connect.try_lock()) {
        	
		if(!is_usb_connected()){			
			usb_connect();
		}

        mutex_try_connect.unlock();
        return true;
    }

    return false;
}

void ManagerT::notify(udata_t* d, uevent_t* e){
    bool available = false;

    if(d) {
        q_evt.put(*d);
        available = true;
    }else if(e){
        udata_t udata;

        udata.type = udata_type_uevent;
        udata.uevent.body = *e;

        q_evt.put(udata);
        available = true;
    }

    if(available) {
        SetEvent(event_notify);
    }
}

void ManagerT::notify_event(uint8_t major, uint8_t minor, uint8_t index){
    bool consumed = false;

    if(major == umajor_manager && minor == manager_minor_ind_core_wm_disc) {
        if(detach_flags & (1<<index)) {
            consumed = true;
        }

        detach_flags &= (~(1<<index));
    }

    if(!consumed) {
        uevent_t event;
    
        event.major = major;
        event.minor = minor;
        event.num_params = 1;
        uparam_t* u = setup_uparam(event.param, 1, 0);
        
        uparam_set(u[1], u8, index);
    
        notify(0, &event);
    }
}

DWORD ManagerT::peek(DWORD timeout){
    void* a[3];

    if(q_evt.size()) {
        /* event is not empty */
        return WAIT_OBJECT_0;
    }else{
        ResetEvent(event_notify);
    }

    a[0] = event_notify;
    a[1] = event_break;
    a[2] = event_disconnected;

    return WaitForMultipleObjects(3, a, false, timeout);
}

bool ManagerT::peek_break(DWORD timeout){
    return peek(timeout) == (WAIT_OBJECT_0+1);
}

int ManagerT::recv(udata_t* udata, uint8_t major, uint8_t minor, uint32_t timeout){
    bool available;

    do{
        available = false;

        if(udata) {
            udata_t V;

            if(q_evt.get(V)) {
                if(((V.type == udata_type_uevent)  &&
                    (V.uevent.body.major == major) &&
                    (V.uevent.body.minor == minor)) 
                   ||
                   (major == 0 && minor == 0)) 
                {
                    if(udata) {
                        *udata = V;
                    }
                    available = true;
                }
            }
        }

        if(available) {
            return ERR_OK;
        }

        if(timeout == 0) {
            /* no wait */
            return ERR_EMPTY;
        }

        ResetEvent(event_notify);
        ResetEvent(event_disconnected);

        switch(peek(timeout)) {
        case WAIT_OBJECT_0:
            break;

        case WAIT_OBJECT_0+1:
            return ERR_BROKEN;

        case WAIT_OBJECT_0+2:
            return ERR_DEVICE_ABSENT;

        case WAIT_TIMEOUT:
            return ERR_TIMEOUT;

        case WAIT_FAILED:
            return ERR_NG1;

        default:
            return ERR_NG2;
        }

    }while(1);
}

int ManagerT::recv_bt_rx(udata_t* udata, uint8_t index, uint8_t channel, uint32_t timeout){	
    do{		
        int r = recv(udata, 0, 0, timeout);
        if(r == ERR_OK) {
            if((udata->type == udata_type_bt_rx) &&
               (udata->usb_bt.index == index)    &&
               (udata->usb_bt.data[0] == channel)){
                return ERR_OK;
            }
        }else{
            return r;
        }
    }while (1);
}

bool ManagerT::core_to_wiipc  (int __core  index, int* mapped_index){
    int i = c2w[index];

    if(i) {
        if(mapped_index) {
            *mapped_index = i - 1;
        }
        return true;
    }

    return false;
}

bool ManagerT::wiipc_to_core  (int __wiipc index, int* mapped_index){
    int i = w2c[index];

    if(i) {
        if(mapped_index) {
            *mapped_index = i - 1;
        }
        return true;
    }

    return false;
}

bool ManagerT::is_wm_connected(int __wiipc index){	
    return (status.conn_bitmap & (1<<index)) != 0;
}


void ManagerT::map_core_wiipc(void){
    int j = 1;
    int i, k;

    for(i=0; i<4; ++i) {
        c2w[i] = 0;
        w2c[i] = 0;

		if(g_wiimote_sources[i] == 2 /* WIIMOTE_SRC_REAL */ || (g_wiimote_sources[i] == 4 && i == 1) /* WIIMOTE_SRC_WII_BALANCE_BOARD */){
            c2w[i] = j++;  
        }
    }

    for(i=0; i<4; ++i) {
        k = c2w[i];
        if(k) {
            w2c[k-1] = i+1;
        }
    }
}

void cb_wm_srcs_changed(void){
    //dprintf("cb_wm_srcs_changed\n");
    Manager()->map_core_wiipc();
}

ManagerT::ManagerT()
:ThreadBaseT(false /* high_prio */), 
    q_evt(), 
    mutex_try_connect(),
    event_notify        (CreateEvent(NULL, 1 /* manual */, 0 /* init */, _T(""))),
    event_disconnected  (CreateEvent(NULL, 1 /* manual */, 0 /* init */, _T(""))),
    usb_handle(), 
    usb_dev_path(), 
    state(Disconnected),
    detach_flags()
{
    wm_srcs_changed_callback = cb_wm_srcs_changed;
    map_core_wiipc();
}

void ManagerT::func(void){
    state_t old_state = Invalid;
    bool    transition = true;
    udata_t udata = {0};

    while(started) {
        old_state = state;

        switch(state) {
        case Disconnected:
            {
                dprintf("Disconnected \n");				

                usb_try_connect();							

                if(is_usb_connected()){
                    //Wiimote::CoreDisplayMessage("WiiPC Connected", 3000);											
                    if(update_status()){						
                        if(status.bridge_mode) {
                            enable_bridge();
                        }else{
                            disable_bridge();
                        }
                        state = status.bridge_mode ? Bridging : Unbridging;
                    }
                }else{										
                    recv(0, 0, 0, 1000);
                }
            }
            break;

        case Unbridged:
            {
                dprintf("Unbridged ");				

				if(transition) {
                    ReaderResetAll();
                    WriterResetAll();
                }

                int r = recv(&udata, 0, 0, INFINITE);

                if(r == ERR_OK) {
                    /* event received */
                    if(udata.type == udata_type_uevent) {
                        switch(UeventType(&udata.uevent.body)) {
                        case Utype(ucore, cfm_bridged):
                            {
                                state = Bridging;
                            }
                            break;
                        }
                    }
                }else if(r == ERR_DEVICE_ABSENT){
                    state = Disconnecting;
                }
            }
            break;

        case Bridged:
            {
                dprintf("Bridged ");				

                int r = recv(&udata, 0, 0, INFINITE);

                if(r == ERR_OK){
                    if(udata.type == udata_type_uevent) {
                        switch(UeventType(&udata.uevent.body)) {
                        case Utype(ucore, cfm_unbridged):
                            {
                                state = Unbridging;
                            }
                            break;

                        case Utype(hidhost, connected):
                            {
                                int j;
                                uint8_t index = udata.uevent.body.param[1].u.u8;
                                dprintf("<connected> ");    								
                                update_status();
                                    
                                if(wiipc_to_core(index, &j)) {
                                    detach_core_wiimote(j);
                                    Sleep(500);
                                    attach_core_wiimote(j);
                                }
                            }
                            break;

                        case Utype(hidhost, disconnected):
                            {
                                dprintf("<disconnected> ");

                                uint8_t index  = udata.uevent.body.param[1].u.u8;
                                int j;
                                
                                update_status();

                                if(wiipc_to_core(index, &j)){
                                    detach_core_wiimote(j);
                                }
                            }
                            break;

                        case Utype(manager, ind_core_wm_disc):
                            {
                                dprintf("<core_wm_disc> ");

                                uint8_t __core I = udata.uevent.body.param[1].u.u8;
                                int J;
                                
                                if(MapperCoreToWiiPC(I, &J)) {
                                    if(is_wm_connected(J)) {
                                        WriterReset(I);
                                        ReaderReset(I);										
                                        
										wiimote_set_channel(I, 0x30, 1, 1);
										attach_core_wiimote(I);																						
                                        //wiimote_disconnect(J);

										// Test attach_core_wiimotes() function;
										//attach_core_wiimotes();
										
										//detach_core_wiimote(I);
										//Sleep(500);
										//attach_core_wiimote(I);
                                    }
                                }
                            }
                            break;
                        }
                    }
                }else if(r == ERR_DEVICE_ABSENT){
                    state = Disconnecting;
                }
            }
            break;

        case Disconnecting:
            {
                dprintf("Disconnecting ");
				
                if(is_usb_connected()) {
                    usb_disconnect();
                }

                state = Disconnected;
            }
            break;

        case Unbridging:
            {
                dprintf("Unbridging ");

                detach_core_wiimotes();
                pause_game();

                if(is_usb_connected()) {
					// Pary Ori Code
                    //update_status();										
                    //state = status.bridge_mode ? Bridging : Unbridged;

					// Webb
					state = Unbridged;
                }else{
                    state = Disconnecting;
                }
            }
            break;

        case Bridging:
            {
                dprintf("Bridging ");

                update_status();

                if(set_all_wiimotes_to_basic()){
                    play_leds1();
                }

                if(is_usb_connected()) {
                    if(status.bridge_mode) {
                        resume_game();
                        Wiimote::CoreBringToFront();
                        detach_core_wiimotes();
                        Sleep(500);
                        attach_core_wiimotes();
                        state = Bridged;
                    }else{
                        state = Unbridging;
                    }
                }else{
                    state = Disconnecting;
                }
            }
            break;
        }

        transition = (old_state != state);

        #if 0
        if(transition) {
            if(state & (Bridging | Unbridging | Disconnecting)) {
            }
        }
        #endif

        old_state = state;
    }

    usb_disconnect();
}

} /* namespace WiiPC */

