#ifndef __WIIPCMANAGER_H__
#define __WIIPCMANAGER_H__

#include "WiiPCThreadBase.h"
#include "QUdata.h"
#include "QReport.h"
#include "udata.h"
#include "WiiPCCommon.h"
#include "../../Core/Src/HW/Wiimote.h"
#include "HidDInfo.h"
#include "TsVstring.h"

namespace WiiPC {

struct ManagerT: public ThreadBaseT {
    typedef enum {
        Invalid       = 0,
        Disconnected  = (1<<0),
        Unbridged     = (1<<1),
        Bridged       = (1<<2),

        Unbridging    = (1<<3),
        Bridging      = (1<<4),
        Disconnecting = (1<<5)
    } state_t;

    state_t     state;
    q_udata_t   q_evt;  /* events */

    std::mutex  mutex_try_connect;
    void*       event_notify;
    void*       event_disconnected;
    void*       usb_handle;
    std::string usb_dev_path;
    uint32_t    detach_flags;

    struct status_t {
        uint8_t bridge_mode;
        uint8_t conn_bitmap;
        uint8_t ir_status;
        uint8_t ir_pos;
        uint8_t discovering;
    };

    status_t status;

    int c2w[MAX_WIIMOTES];
    int w2c[MAX_WIIMOTES];

    ManagerT();
	
    bool update_status  (void);
    bool wait_recv_bt_rx(int __wiipc index, uint8_t channel, uint32_t timeout);

    bool set_wiimote_to_basic       (int __wiipc index, bool wait_ack = false);
    bool set_all_wiimotes_to_basic  (void);

    void detach_core_wiimote  (int __core index);
    void attach_core_wiimote    (int __core index);

    void detach_core_wiimotes (void);
    void attach_core_wiimotes   (void);

    void warn_bridged_no_slot   (void);

    void pause_game (void);
    void resume_game(void);
    bool play_leds1 (void);
    bool play_leds2 (int index);

    virtual void func    (void);
    bool  is_usb_connected   (void);
    bool  is_bridged     (void);
    bool  is_usb_disconnected(void);


    void  usb_disconnect (void);
    void  usb_connect    (void);
    bool  usb_try_connect(void);
    void  notify         (udata_t* d, uevent_t* e);
    void  notify_event   (uint8_t major, uint8_t minor, uint8_t index);

    DWORD peek           (DWORD timeout);
    bool  peek_break     (DWORD timeout);
    int   recv           (udata_t* udata, uint8_t major, uint8_t minor, uint32_t timeout);
    int   recv_bt_rx     (udata_t* udata, uint8_t index, uint8_t channel, uint32_t timeout);

    void  map_core_wiipc(void);

    bool  core_to_wiipc  (int __core  index, int* mapped_index);
    bool  wiipc_to_core  (int __wiipc index, int* mapped_index);

    bool  is_wm_connected(int __wiipc index);

};

} /* namespace WiiPC */

#endif /* __WIIPCMANAGER_H__ */
