#include "WiiPCMapper.h"
#include "dconsole.h"

namespace WiiPC {

void MapperT::init(unsigned int _g_wiimote_sources[4], uint8_t* _conn_bitmap){
    reset();
    wiimote_sources = (unsigned int*)_g_wiimote_sources;
    conn_bitmap = _conn_bitmap;

    for(int i=0; i<4; ++i){
        old_wiimote_sources[i] = wiimote_sources[i];
    }
    old_conn_bitmap = *conn_bitmap;
    is_inited = true;
}

uint8_t MapperT::core_bitmap(void){
    uint8_t bitmap = 0;
    for(int i=0; i<4; ++i) {
        if(wiimote_sources[i] & 2 /* WIIMOTE_SRC_REAL */){
            bitmap |= (1<<i);
        }
    }

    return bitmap;
}

uint8_t MapperT::wiipc_bitmap(void){
    return *conn_bitmap;
}

void MapperT::dump(void){
    if(!is_inited) {
        //dprintf("not inited\n");
        return;
    }

    //dprintf("\nwiimote_sources=%d %d %d %d\n",
    //        wiimote_sources[0],
    //        wiimote_sources[1],
    //        wiimote_sources[2],
    //        wiimote_sources[3]);


    uint8_t cbitmap = core_bitmap();

    //dprintf("\ncbmp=");
    for(int i=0; i<4; ++i) {
        if(cbitmap & (1<<i)) {
            //dprintf("1");
        }else{
            //dprintf("0");
        }
    }
    //dprintf("\n");
    //dprintf("c2w=%d %d %d %d\n", c2w[0], c2w[1], c2w[2], c2w[3]);
    //dprintf("w2c=%d %d %d %d\n", w2c[0], w2c[1], w2c[2], w2c[3]);
}

void MapperT::reset(void){
    for(int i=0; i<4; ++i) {
        c2w[i] = 0;
        w2c[i] = 0;
    }
}

bool __valid MapperT::wiipc_to_core(int __wiipc index, int* __core  mapped_index){
    int I;
    bool valid = false;

    I = w2c[index];

    if(I) {
        valid = true;

        if(mapped_index) {
            *mapped_index = I-1;
        }
    }

    return valid;
}

bool __valid MapperT::core_to_wiipc(int __core  index, int* __wiipc mapped_index){
    int I;
    bool valid = false;

    I = c2w[index];

    if(I) {
        valid = true;

        if(mapped_index) {
            *mapped_index = I-1;
        }
    }

    return valid;
}

int MapperT::num_core_mapped(void){
    int n = 0;
    for(int i=0; i<4; ++i){
        if(c2w[i]) {
            n++;
        }
    }

    return n;
}

int MapperT::num_wiipc_mapped(void){
    int n = 0;
    for(int i=0; i<4; ++i) {
        if(w2c[i]) {
            n++;
        }
    }

    return n;
}

void MapperT::remap(void){
    reset();

    if(!is_inited) return;

    uint8_t cbitmap = core_bitmap();
    uint8_t wbitmap = wiipc_bitmap();

    for(int i=0; i<4; ++i) {
        if(cbitmap & (1<<i)) {
            for(int j=0; j<4; ++j) {
                if(wbitmap & (1<<j)) {
                    c2w[i] = (j+1);
                    w2c[j] = (i+1);
                    wbitmap &= (~(1<<j));
                    break;
                }
            }
        }
    }
}

#if 0
int __from_one_but_zero MapperT::notify_connected(int __from_zero index){
    if(index >= 4) {
        return 0;
    }

    uint8_t cbitmap = core_bitmap();

    for(int i=0; i<4; ++i) {
        if(cbitmap & (1<<i)) {
            if(c2w[i] == 0) {
                /* available */
                w2c[index] = i+1;
                c2w[i]     = index +1;
                return i + 1;
            }
        }
    }

    return 0;
}

int __from_one_but_zero MapperT::notify_disconnected(int __from_zero index){
    if(index >=4 ) {
        return 0;
    }

    int I = 0;

    for(int i =0; i<4; ++i) {
        if(c2w[i] == (index+1)) {
            c2w[i] = 0;
            I = i;
        }
    }

    w2c[index] = 0;

    return I;
}
#endif

MapperT::MapperT(): wiimote_sources(), conn_bitmap(), is_inited(){
    reset();
}

} /* namespace WiiPC */

