#include "WiiPCReader.h"
#include "dconsole.h"

namespace WiiPC {

void ReaderT::func(void) {
    while(started) {
        DWORD r;

        r = try_read(INFINITE);
        if(r == ERR_DEVICE_ABSENT){
            ManagerUsbDisconnect();
            break;
        }
        if(r == ERR_BROKEN){
            break;
        }
    }
}

void ReaderT::on_ucore(udata_t* d, uevent_t* e){
    if(d) {
        if(ManagerIsBridged()) {
            //std::lock_guard<std::mutex> lk(mutex_q_feed);
            int i, j;
            uint8_t bitmap = d->usb_extra_bt.bitmap;            

            for(j=0; j<4; ++j) {
                if(bitmap & (1<<j)) {
                    if(MapperWiiPCToCore(j, &i)){
                        report_t r;
                        memcpy(r.data, d->usb_extra_bt.data[j], 22);
                        uint8_t ch = r.data[0];
                        
                        /* filter out HOME */    
                        r.data[2] &= (~0x80);

                        if(ch < 0x30) {
                            q_ctl[i].put(r);
                        }else{
                            q_int[i].put(r);
                        }                        

                        #if 1
                        //print wiimotes' output report                        
                        if (r.data[0] >= 0x20 && r.data[0] <= 0x2f && j == 0) {
                            //dprintf("D[%x] = [",j);
                            //dprintbuf(r.data, 22);
                            //dprintf("]\n");           
                        }
                        #endif                                                            
                    }else{
                        //dprintf("invaid mapping from %d\n", j);
                    }
                }
            }
        }else{
            ManagerNotify(d, 0);
        }
    }else if(e){
        ManagerNotify(0, e);
    }
}

void ReaderT::on_info(uint8_t channel, uint8_t* u){
    /* drop */
}

void ReaderT::proceed(udata_t* _udata){
    if(udata.usb.usb_report_id == USB_EXTRA_TX_REPORT_ID){
        switch(udata.usb_extra.cmd){
            case udata_usb_extra_cmd_bt_rx:
                {
                    udata.type = udata_type_bt_rx;

                    if(_udata) {
                        *_udata = udata;
                        return;
                    }

                    on_ucore(&udata, 0);
                }
                break;

            case udata_usb_extra_cmd_uevent:
                {
                    {
                        std::lock_guard<std::mutex> lk(mutex_transx_vars);
                        if(transx_waiting){
                            if(udata.uevent.body.major == transx_major &&
                               udata.uevent.body.minor == transx_minor){
                               uparam_t* u = SetupUevent(&transx_udata.uevent.body,
                                                          udata.uevent.body.major,
                                                          udata.uevent.body.minor,
                                                          udata.uevent.body.num_params,
                                                          udata.uevent.body.param[0].u.head.len_tail);

                               ntoh_uparam((const uint8_t*)(&udata.uevent.body.param[0]), u);

                               transx_udata.type = udata_type_uevent;
                               transx_received = true;
                               break;
                            }
                        }
                    }
                    {
                        uparam_t* u = SetupUevent(&uevent,
                                                   udata.uevent.body.major,
                                                   udata.uevent.body.minor,
                                                   udata.uevent.body.num_params,
                                                   udata.uevent.body.param[0].u.head.len_tail);

                        ntoh_uparam((uint8_t*)udata.uevent_raw.raw, u);

                        //dprintf("on_ucore(%d,%d) ", uevent.major, uevent.minor);

                        if(uevent.major == umajor_ucore){
                            if(uevent.minor == ucore_minor_cfm_bridged) {
                                //dprintf("<bridged> ");
                            }else if(uevent.minor == ucore_minor_cfm_unbridged){
                                //dprintf("<unbridged> ");
                            }
                        }
                        on_ucore(0, &uevent);
                    }
                }
                break;

            case udata_usb_extra_cmd_info:
                {
                    on_info(udata.usb_extra.param[0], udata.usb_extra.param + 1);
                }
                break;
        }
    }else{
        /* drop */
        //dprintf("DROP ");
    }
}


int ReaderT::try_read(DWORD timeout, udata_t* _udata){
    if(mutex_try.try_lock()) {
        DWORD r;
        r = read(timeout);

        if(r == ERR_OK) {
            proceed(_udata);
        }
        mutex_try.unlock();

        return r;
    }

    return ERR_DOING_ALREADY;
}

int ReaderT::read(DWORD timeout){
    DWORD b = 0, r;
    HANDLE a[2];
    int result = ERR_NG2;

    if(ManagerIsUsbDisconnected()) {
        return ERR_DEVICE_ABSENT;
    }

    ResetEvent(overlap.hEvent);

    memset(&udata.raw, 0, sizeof(udata_t));

    if(ReadFile(ManagerUsbHandle(), &udata.raw[2], usb_payload_size, &b, &overlap) == 0){
        b = GetLastError();

        if(b == ERROR_IO_PENDING){
            a[0] = overlap.hEvent;
            a[1] = event_break;

            r = WaitForMultipleObjects(2, a, false, timeout);

            switch(r){
            case WAIT_OBJECT_0:
                result = ERR_OK;
                break;
            case WAIT_OBJECT_0+1:
                result = ERR_BROKEN;
                break;
            case WAIT_TIMEOUT:
                result = ERR_TIMEOUT;
                break;
            case WAIT_FAILED:
                result = ERR_NG1;
                break;
            }
        }else if((b == ERROR_HANDLE_EOF) || (b == ERROR_DEVICE_NOT_CONNECTED)){
            result = ERR_DEVICE_ABSENT;
        }
        return result;
    }else {
        return b == (DWORD)usb_payload_size ? ERR_OK /* OK */ : ERR_NG3 /* NG */;
    }
}

bool ReaderT::feed(report_t* report, int __core index){
    int j;

    if(MapperCoreToWiiPC(index, &j)) {
        if(ManagerIsWmConnected(j)) {
        }else{
            return false;
        }
    }else{
        return false;
    }

    if(q_ctl[index].get(*report)) {
        return true;
    }

    if(q_int[index].get_and_clear(*report)){
        memcpy(d_int[index].data, report->data, 22);
        return true;
    }

    *report = d_int[index];

    return true;
}

void ReaderT::reset(int __core index){
    q_int[index].clear();
    q_ctl[index].clear();
    memcpy(d_int[index].data, "\x30\x00\x00", 3);
}

void ReaderT::reset_all(void){
    for(int i=0; i<4; ++i) {
        reset(i);
    }
}



ReaderT::ReaderT()
    :ThreadBaseT(true /* high_prio */, 1), udata(), uevent()
{
    memset(d_int, 0, sizeof(d_int));
    reset_all();
}

} /* namespace WiiPC */
