#ifndef __WIIPCTHREADBASE_H__
#define __WIIPCTHREADBASE_H__

#include "StdThread.h"
#include "StdMutex.h"

namespace WiiPC {

struct ThreadBaseT {
    std::thread thread;
    bool started;
    bool left;
    bool high;
    std::mutex mutex_vars;
    std::mutex mutex_try;
    OVERLAPPED overlap;
    void*      event_break;
    int  did; /* debug id */

    ThreadBaseT(bool high_prio = false, int _did = 0)
        :thread(), started(), left(), high(high_prio), 
         mutex_vars(), mutex_try(), did(_did),
        event_break(CreateEvent(NULL, 1, 1, _T("")))
    {
        overlap.hEvent = CreateEvent(NULL, 1, 1, _T(""));
        overlap.Offset = 0;
        overlap.OffsetHigh = 0;
    }

    virtual ~ThreadBaseT(){
    }

    virtual void func(void) = 0;

    void loop(void){
        started = true;
        left = false;

        func();

        started = false;
        left    = true;
    }

    void start(void){
        ResetEvent(event_break);

        if(!started){
            thread = std::thread(std::mem_fun(&WiiPC::ThreadBaseT::loop), this);
            if(high) {
                SetThreadPriority(thread.native_handle(), THREAD_PRIORITY_TIME_CRITICAL);
            }

            while(!started){
                Sleep(10);
            }
        }
    }

    void stop(void){
        if(started) {
            mutex_vars.lock();
            started = false;
            left = false;
            mutex_vars.unlock();
            SetEvent(event_break);

            while(!left) {
                Sleep(10);
            }

            started = false;
            ResetEvent(event_break);
        }
    }

    void forced_stop(void){
        left = false;
        started = false;
        SetEvent(event_break);

        thread.join();

        while(!left) {
            Sleep(10);
        }
    }
};

} /* namespace WiiPC */

#endif /* __WIIPCTHREADBASE_H__ */
