/*
    Akar CC Ltd 2013
    File IO
*/
#ifndef __IOFILE_H__
#define __IOFILE_H__

#include <stdio.h>
#include <string>
#include "CommonTypes.h"

class IOFile
{
public:
	IOFile();
	IOFile(std::FILE* file);
	//IOFile(const std::string& filename, const char openmode[]);
    IOFile(const std::wstring& filename, const char openmode[]);

	~IOFile();
	
	IOFile(IOFile&& other);
	IOFile& operator=(IOFile&& other);
	
	void Swap(IOFile& other);

	//bool Open(const std::string& filename, const char openmode[]);
    bool Open(const std::wstring& filename, const char openmode[]);
	bool Close();

	template <typename T>
	bool ReadArray(T* data, size_t length)
	{
		if (!IsOpen() || length != std::fread(data, sizeof(T), length, m_file))
			m_good = false;

		return m_good;
	}

	template <typename T>
	bool WriteArray(const T* data, size_t length)
	{
		if (!IsOpen() || length != std::fwrite(data, sizeof(T), length, m_file))
			m_good = false;

		return m_good;
	}

	bool ReadBytes(void* data, size_t length)
	{
		return ReadArray(reinterpret_cast<char*>(data), length);
	}

	bool WriteBytes(const void* data, size_t length)
	{
		return WriteArray(reinterpret_cast<const char*>(data), length);
	}

	bool IsOpen() { return NULL != m_file; }

	// m_good is set to false when a read, write or other function fails
	bool IsGood() {	return m_good; }
	operator void*() { return m_good ? m_file : NULL; }

	std::FILE* ReleaseHandle();

	std::FILE* GetHandle() { return m_file; }

	void SetHandle(std::FILE* file);

	bool Seek(s64 off, int origin);
	u64 Tell();
	u64 GetSize();
	bool Resize(u32 size);
	bool Flush();

	// clear error state
	void Clear() { m_good = true; std::clearerr(m_file); }

private:
	IOFile(const IOFile&) /*= delete*/;
	IOFile& operator=(const IOFile&) /*= delete*/;

	std::FILE* m_file;
	bool m_good;
};


#endif /* __IOFILE_H__ */
