/*
    Unified Data
*/
#ifndef __UDATA_H__
#define __UDATA_H__

#include "uparam.h"

typedef enum {
    udata_type_nop = 0,
    udata_type_usb_rx,
    udata_type_bt_rx,

    udata_type_usb_rx_set_report,

    udata_type_uevent = 0x90,
    udata_type_end
} udata_type_enum_t;

#define USB_EXTRA_RX_REPORT_ID         0x10
#define USB_EXTRA_TX_REPORT_ID         0x21

/* for usb_report_id = 0x21 */
typedef enum {
    udata_usb_extra_cmd_nop = 0,
    udata_usb_extra_cmd_bt_tx,
    udata_usb_extra_cmd_bt_rx,
    udata_usb_extra_cmd_uevent,

    udata_usb_extra_cmd_enable_bridge,
    udata_usb_extra_cmd_disable_bridge,

    udata_usb_extra_cmd_status,
	//udata_usb_extra_cmd_command,
    
    udata_usb_extra_cmd_info,   /* unidirectional information from module to client */

    udata_usb_extra_cmd_present,
    udata_usb_extra_cmd_absent,
    udata_usb_extra_cmd_test,
    
    udata_usb_extra_cmd_end
} udata_usb_extra_cmd_t;

#pragma pack(push, 1)

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         cmd;
} udata_head_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         payload[1];
} udata_usb_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         cmd;
    uint8_t         param[1];
} udata_usb_extra_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         cmd;
    uint8_t         bitmap;
    uint8_t         data[4][22];
} udata_usb_extra_bt_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         cmd;
    uint8_t         index;
    uint8_t         len;
    uint8_t         data[1];
} udata_bt_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         cmd;
    uint8_t         index;
    uint8_t         len;
    uint8_t         data[1];
} udata_usb_bt_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         cmd;
    uint8_t         bitmap;
    uint8_t         data[4][22];
} udata_usb_bt_acc_t;

typedef struct {
    uint8_t             type;
    uint8_t             counter;
    uint8_t             usb_report_id;
    uint8_t             cmd;
    uint8_t             index;
    uint8_t             len;
    uint8_t             report[1];
} udata_bt_wiimote_t;

typedef struct {
    uint8_t             type;
    uint8_t             counter;
    uint8_t             usb_report_id;
    uint8_t             cmd;
    uint8_t             index;
    uint8_t             len;
    uint8_t             raw[1];
} udata_bt_wiimote_raw_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         length;
    uint8_t         payload[1];
} udata_usb_set_report_t;


typedef struct uevent_t {
    uint8_t         major;
    uint8_t         minor;
    uint8_t         num_params;
    uparam_t        param[24];
} uevent_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         cmd;
    uevent_t        body;
} udata_uevent_t;

typedef struct {
    uint8_t         type;
    uint8_t         counter;
    uint8_t         usb_report_id;
    uint8_t         cmd;
    uint8_t         major;
    uint8_t         minor;
    uint8_t         num_params;
    uint8_t         raw[1];
} udata_uevent_raw_t;

typedef union udata_t {
    uint8_t                     type;
    udata_head_t                head;
    uint8_t                     raw[93];
    udata_usb_t                 usb;
    udata_usb_extra_t           usb_extra;
    udata_usb_extra_bt_t        usb_extra_bt;
    udata_bt_t                  bt;
    udata_usb_bt_t              usb_bt;
    udata_usb_bt_acc_t          usb_bt_acc;
    udata_uevent_t              uevent;
    udata_uevent_raw_t          uevent_raw;
    udata_bt_wiimote_t          wiimote;
    udata_bt_wiimote_raw_t      wiimote_raw;
    udata_usb_set_report_t      usb_set_report;
} udata_t;

#pragma pack(pop)

#endif /* __UDATA_H__ */
