/*
    Akar CC Ltd 2012
    UParam
*/
#include "uparam.h"
//#include "zmalloc.h"
#include <string.h>

static uint16_t    cal_bytes(uint8_t num_params, uint8_t len_tail){
    return sizeof(uparam_t) * (num_params + 1) + len_tail;
}

uparam_t*   setup_uparam(void* ptr, uint16_t nparams, uint8_t nb_tail){
    uparam_t* u = (uparam_t*)ptr;

    u->type = uparam_type_head;
    u->u.head.num_params = (uint8_t)nparams;
    u->u.head.len_tail   = nb_tail;

    return u;
}

uint16_t    copy_uparam(uparam_t* dest, const uparam_t* src){
    uint16_t n = cal_bytes(src->u.head.num_params, src->u.head.len_tail);
    memcpy(dest, src, n);

    return n;
}

#if 0
uparam_t*   alloc_uparam(uint16_t num_params, uint8_t len_tail){
    uparam_t* u = (uparam_t*)zmalloc(cal_bytes(num_params, len_tail));

    u->type = uparam_type_head;
    u->u.head.num_params = num_params;
    u->u.head.len_tail   = len_tail;
    return u;
}

void        free_uparam(void* uparam){
    if(uparam) {
        zfree(uparam);
    }
}


static uparam_t*   alloc_uparam_raw(const uint8_t* raw){
    uparam_t* u = (uparam_t*)zmalloc(cal_bytes(raw[1] /* nparams */, raw[2] /* nb_tail */));

    u->type = uparam_type_head;
    u->u.head.num_params = raw[1];
    u->u.head.len_tail   = raw[2];
    
    return u;
}
#endif

void uparam_set_value_ptr     (uparam_t* u, void*  value){
    u->type  = uparam_type_ptr;
    u->u.ptr = value;
}

void uparam_set_value_u8      (uparam_t* u, uint8_t  value){
    u->type  = uparam_type_u8;
    u->u.u8  = value;
}

void uparam_set_value_i8      (uparam_t* u, int8_t   value){
    u->type  = uparam_type_i8;
    u->u.i8  = value;
}

void uparam_set_value_u16     (uparam_t* u, uint16_t value){
    u->type  = uparam_type_u16;
    u->u.u16 = value;
}

void uparam_set_value_i16     (uparam_t* u, int16_t  value){
    u->type  = uparam_type_i16;
    u->u.i16 = value;
}

void uparam_set_value_u32     (uparam_t* u, uint32_t value){
    u->type  = uparam_type_u32;
    u->u.u32 = value;
}

void uparam_set_value_i32     (uparam_t* u, int32_t  value){
    u->type  = uparam_type_i32;
    u->u.i32 = value;
}

void uparam_set_value_boolean (uparam_t* u, boolean  value){
    u->type      = uparam_type_bl;
    u->u.bl = value;
}

void uparam_set_tail_bdaddr (uparam_t* u, const BdAddr_t* value){
    u->type = uparam_type_tail_bdaddr;
    *((BdAddr_t*)(u[u[0].u.head.num_params].u.tail)) = *value;
}

void uparam_get_tail_bdaddr (uparam_t* u, BdAddr_t* result){
    *result = *((BdAddr_t*)(u[u[0].u.head.num_params].u.tail));
}

uint8_t*    uparam_set_tail_zeros(uparam_t* u, uint8_t len){
    uint8_t num_params;
    uint8_t len_tail;

    num_params         = u[0].u.head.num_params;
    len_tail           = u[0].u.head.len_tail;
    u[num_params].type = uparam_type_tail_str;

    if(len > len_tail) {
        len = len_tail;
    }

    memset(u[num_params].u.tail, 0, len);
    return u[num_params].u.tail;
}

void        uparam_set_tail_str(uparam_t* u, const uint8_t* src, uint8_t len){
    uint8_t num_params;
    uint8_t len_tail;

    num_params         = u[0].u.head.num_params;
    len_tail           = u[0].u.head.len_tail;
    u[num_params].type = uparam_type_tail_str;

    if(len > len_tail) {
        len = len_tail;
    }

    memcpy(u[num_params].u.tail, src, len);
}

void        uparam_get_tail_str(const Uparam u, uint8_t* dest, uint8_t len){
    uint8_t num_params;
    uint8_t len_tail;

    num_params = u[0].u.head.num_params;
    len_tail = u[0].u.head.len_tail;

    if(len < len_tail) {
        len_tail = len;
    }

    memcpy(dest, u[num_params].u.tail, len_tail);
}

uint16_t    uparam_get_tail_len(uparam_t* u){
    if(u) {
        return u[0].u.head.len_tail;
    }

    return 0;
}

uint8_t*    uparam_get_tail_ptr(uparam_t* u){
    if(u) {
        return u[u[0].u.head.num_params].u.tail;
    }

    return 0;
}

uint16_t  hton_uparam(uparam_t* uparam, uint8_t* dest){
    uint16_t i = 0;
    uint16_t len;
    uint8_t  len_tail;

    *dest++ = uparam_type_head;

    len = uparam[0].u.head.num_params;
    *dest++ = (uint8_t)len;
    len_tail = uparam[0].u.head.len_tail;
    *dest++ = len_tail;

    uparam ++;
    
    while(len) {

        switch(uparam->type) {
        case uparam_type_ptr:
            {
                uint16_t v = (uint16_t)(uparam->u.ptr);

                *dest++ = uparam_type_ptr;
                *dest++ = (v & 0xFF);
                v >>= 8;
                *dest++ = (v & 0xFF); 
            }
            break;
    
        case uparam_type_u8:
            {
                *dest ++ = uparam_type_u8;
                *dest ++ = uparam->u.u8;
            }
            break;

        case uparam_type_bl:
            {
                *dest ++ = uparam_type_bl;
                *dest ++ = uparam->u.bl;
            }
            break;

        case uparam_type_i8:
            {
                *dest ++ = uparam_type_i8;
                *dest ++ = uparam->u.i8;
            }
            break;
    
        case uparam_type_u16:
            {
                uint16_t v = uparam->u.u16;

                *dest++ = uparam_type_u16;
                *dest++ = (v & 0xFF);
                v >>= 8;
                *dest++ = (v & 0xFF); 
            }
            break;

        case uparam_type_i16:
            {
                int16_t v = uparam->u.i16;

                *dest++ = uparam_type_i16;
                *dest++ = (v & 0xFF);
                v >>= 8;
                *dest++ = (v & 0xFF);
            }
            break;
    
        case uparam_type_u32:
            {
                uint32_t v = uparam->u.u32;

                *dest++ = uparam_type_u32;

                *dest++ = (v & 0xFF);   /* 0 */
                v >>= 8;
                *dest++ = (v & 0xFF);   /* 1 */
                v >>= 8;
                *dest++ = (v & 0xFF);   /* 2 */
                v >>= 8;
                *dest++ = (v & 0xFF);   /* 3 */
            }
            break;

        case uparam_type_i32:
            {
                int32_t v = uparam->u.i32;

                *dest++ = uparam_type_i32;

                *dest++ = (v & 0xFF);   /* 0 */
                v >>= 8;
                *dest++ = (v & 0xFF);   /* 1 */
                v >>= 8;
                *dest++ = (v & 0xFF);   /* 2 */
                v >>= 8;
                *dest++ = (v & 0xFF);   /* 3 */
            }
            break;
    
        case uparam_type_tail_bdaddr:
            {
                BD_ADDR_T addr; // = uparam->u.bdaddr;

                uparam_get_tail_bdaddr(uparam, & addr);

                *dest++ = uparam_type_tail_bdaddr;

                *dest++ = (addr.nap & 0xFF);
                addr.nap >>= 8;
                *dest++ = (addr.nap & 0xFF);
                *dest++ = (addr.uap);

                *dest++ = (addr.lap & 0xFF);
                addr.lap >>= 8;
                *dest++ = (addr.lap & 0xFF);
                addr.lap >>= 8;
                *dest++ = (addr.lap & 0xFF);
                addr.lap >>= 8;
                *dest++ = (addr.lap & 0xFF);

            }
            return i;   /* tail */

    
        case uparam_type_tail_str:
            {
                uint8_t* src = uparam->u.tail;

                *dest++ = uparam_type_tail_str;

                while(len_tail--) {
                    *dest++ = *src++;
                }

            }
            return i; /* tail */

    
        default:
            return i;
        }

        uparam ++;
        i++;
        len --;
    }

    return i;
}

uint16_t    ntoh_uparam(const uint8_t* src, uparam_t* uparam){
    uint16_t i = 0;
    uint8_t type;
    uint8_t len;
    uint8_t len_tail;

    type = *src++;

    if(type != uparam_type_head) {
        return i;
    }

    len = *src++;
    uparam[0].u.head.num_params = len;
    len_tail = *src++;
    uparam[0].u.head.len_tail = len_tail;

    uparam ++;
    
    while(len) {
        switch(*src++){
        case uparam_type_ptr:
            {
				uint16_t v, h0, h1;

				h0 = *src++;
				h1 = *src++;

				v = h0 | (h1<<8);

                uparam->u.ptr = (void*)v;
            }
            break;

        case uparam_type_u8:
            {
                uint8_t v = *src++;
                uparam->u.u8 = v;
            }
            break;

        case uparam_type_bl:
            {
                boolean v = *src++;
                uparam->u.bl = v;
            }
            break;

        case uparam_type_i8:
            {
                int8_t v = *src++;
                uparam->u.i8 = v;
            }
            break;

        case uparam_type_u16:
			{
				uint16_t v, h0, h1;

				h0 = (*src++);
				h1 = (*src++);

				v = (h1 << 8) | h0;

                uparam->u.u16 = v;
            }
            break;

        case uparam_type_i16:
            {
                int16_t v, h0, h1;
                h0 = (*src++);
                h1 = (*src++);

                v = (h1 << 8) | h0;

                uparam->u.i16 = v;
            }
            break;

        case uparam_type_u32:
            {
                uint32_t v, h0, h1, h2, h3;
                h0 = *src++;
                h1 = *src++;
                h2 = *src++;
                h3 = *src++;

                v = h0 | (h1<<8) | (h2<<16) | (h3<<24);
                uparam->u.u32 = v;
            }
            break;

        case uparam_type_i32:
            {
                int32_t v, h0, h1, h2, h3;
                h0 = *src++;
                h1 = *src++;
                h2 = *src++;
                h3 = *src++;

                v = h0 | (h1<<8) | (h2<<16) | (h3<<24);
                uparam->u.i32 = v;
            }
            break;

        case uparam_type_tail_bdaddr:
            {
                BD_ADDR_T bd;
                {
                    uint16_t v, h0, h1;

                    h0 = *src++;
                    h1 = *src++;

                    v = h0 | (h1<<8);
                    bd.nap = v;
                }

                bd.uap = *src++;

                {
                    uint32_t v, h0, h1, h2, h3;

                    h0 = *src++;
                    h1 = *src++;
                    h2 = *src++;
                    h3 = *src++;

                    v = h0 | (h1<<8) | (h2<<16) | (h3<<24);

                    bd.lap = v;
                }

                uparam_set_tail_bdaddr(uparam, & bd);
                //uparam->u.bdaddr = bd;
            }
            return i;   /* tail */

        case uparam_type_tail_str:
            {
                uint8_t* p = uparam->u.tail;
                
                while(len_tail--) {
                    *p++ = *src++;
                }

            }
            return i; /* tail */


        default:
            return i;
        }

        uparam++;
        i++;
        len --;
    }

    return i;
}

#if 0
uparam_t*   ntoh_uparam_raw(const uint8_t* src){
    uparam_t* u = alloc_uparam_raw(src);

    ntoh_uparam(src, u);
    return u;
}
#endif
