#ifndef __UPARAM_H__
#define __UPARAM_H__

#include "akartypes.h"

#ifdef __WIN32__
#pragma pack(push, 1)
#endif /* __WIN32__ */

typedef enum {
    uparam_type_head = 0,
    uparam_type_ptr,    /* pointer */
    uparam_type_bl,     /* boolean */
    uparam_type_u8,
    uparam_type_i8,
    uparam_type_u16,
    uparam_type_i16,
    uparam_type_u32,
    uparam_type_i32,
    uparam_type_tail_str,
    uparam_type_tail_bdaddr,
    uparam_type_end
} uparam_type_t;

typedef struct {
    uint8_t     num_params;
    uint8_t     len_tail;
} uparam_head_t;

typedef union {
    uparam_head_t   head;
    void*           ptr;
    uint8_t         u8;
    int8_t          i8;
    uint16_t        u16;
    int16_t         i16;
    uint32_t        u32;
    int32_t         i32;
    uint16_t        conn;
    uint8_t         bl;
    uint8_t         tail[1];    /* variant length in tail */
 
    /* for macro glue ## */
    void*           _ptr;
    uint8_t         _u8;
    int8_t          _i8;
    uint16_t        _u16;
    int16_t         _i16;
    uint32_t        _u32;
    int32_t         _i32;
    uint8_t         _bl;
} uparam_union_t;

typedef struct {
    uint8_t         type;
    union {
        uparam_union_t  _union;
        uint8_t         value;
    };
} uparam_u8_t;

typedef struct {
    uint8_t         type;
    union {
        uparam_union_t  _union;
        boolean          value;
    };
} uparam_bl_t;

typedef struct {
    uint8_t         type;
    union {
        uparam_union_t  _union;
        int8_t          value;
    };
} uparam_i8_t;

typedef struct {
    uint8_t         type;
    union{
        uparam_union_t  _union;
        uint16_t        value;
    };
} uparam_u16_t;

typedef struct {
    uint8_t         type;
    union{
        uparam_union_t  _union;
        int16_t         value;
    };
} uparam_i16_t;

typedef struct {
    uint8_t         type;

    union {
        uparam_union_t  _union;
        uint32_t        value;
    };
} uparam_u32_t;

typedef struct {
    uint8_t         type;

    union {
        uparam_union_t  _union;
        int32_t         value;
    };
} uparam_i32_t;

typedef struct {
    uint8_t         type;

    union{
        uparam_union_t  _union;
        void*           value;
    };
} uparam_ptr_t;

typedef struct {
    uint8_t         type;

    union{
        uparam_union_t  _union;
        BdAddr_t        value;
    };
} uparam_tail_bdaddr_t;

typedef struct {
    uint8_t         type;

    union{
        uparam_union_t  _union;
        uint8_t         value[1];
    };
} uparam_tail_str_t;

typedef struct {
    uint8_t         type;
    uparam_union_t  u;
} uparam_t;

typedef uparam_t* Uparam;

union  udata_t;
struct uevent_t;

typedef union  udata_t*     Udata;
typedef struct uevent_t*    Uevent;

typedef uint16_t (*filter_t)(Udata udata, Uevent uevent, void* obj);

void uparam_set_value_ptr         (uparam_t* u, void*    value);
void uparam_set_value_u8          (uparam_t* u, uint8_t  value);
void uparam_set_value_i8          (uparam_t* u, int8_t   value);
void uparam_set_value_u16         (uparam_t* u, uint16_t value);
void uparam_set_value_i16         (uparam_t* u, int16_t  value);
void uparam_set_value_u32         (uparam_t* u, uint32_t value);
void uparam_set_value_i32         (uparam_t* u, int32_t  value);
void uparam_set_value_boolean     (uparam_t* u, uint8_t  value);
void uparam_set_tail_bdaddr       (uparam_t* u, const BdAddr_t* value);
void uparam_get_tail_bdaddr       (uparam_t* u, BdAddr_t* result);

uparam_t*   setup_uparam(void* ptr, uint16_t nparams, uint8_t nb_tail);
uint16_t    copy_uparam(uparam_t* dest, const uparam_t* src);

uint16_t    ntoh_uparam (const uint8_t* src, uparam_t* uparam);
uint16_t    hton_uparam (uparam_t* /* in */ uparam, uint8_t* dest);
uparam_t*   ntoh_uparam_raw(const uint8_t* src);

uparam_t*   alloc_uparam(uint16_t num_params, uint8_t len_tail);
void        free_uparam (void* uparam);

uint8_t*    uparam_set_tail_zeros(uparam_t* u, uint8_t len);
void        uparam_set_tail_str(uparam_t* u, const uint8_t* src, uint8_t len);
void        uparam_get_tail_str(const Uparam u, uint8_t* dest, uint8_t len);

uint16_t    uparam_get_tail_len(uparam_t* u);
uint8_t*    uparam_get_tail_ptr(uparam_t* u);

#define uparam_set(U, T, value)           uparam_set_value_##T(&(U), value)

#define utype(major, minor) ((uint16_t)(((major << 8) | (minor & 0xFF))))
#define Utype(M, N)         ((uint16_t)((umajor_##M << 8) | (M##_minor_##N & 0xFF))) 

#ifdef __WIN32__
#pragma pack(pop)
#endif /* __WIN32__ */


#endif /* __UPARAM_H__ */

