
#include "Common.h"

#include "Wiimote.h"
#include "WiimoteReal/WiimoteRealBase.h"

#include "WiimoteEmu/WiimoteEmu.h"
#include "Movie.h"

#include "Core.h"
#include "ConfigManager.h"
#include "ControllerInterface/ControllerInterface.h"

#include "../../InputCommon/Src/InputConfig.h"
#include "dconsole.h"
#include "../IPC_HLE/WII_IPC_HLE_WiiMote.h"
#include "../IPC_HLE/WII_IPC_HLE_Device_usb.h"
#include "WiiPC.h"
wm_sources_changed_callback_t wm_srcs_changed_callback = 0;
wiimote_sources_t  g_wiimote_sources;
namespace Wiimote {
    const unsigned char wiimote_channel_sizes[] = {
        /*00    01    02    03    04    05,   06    07    08    09    0A    0B    0C    0D    0E    0F */
          0x00, 0x01, 0x01, 0x02, 0x05, 0x04, 0x04, 0x04, 0x07, 0x07, 0x07, 0x07, 0x07, 0x05, 0x04, 0x04,
        
        /*10    11    12    13    14    15    16    17    18    19    1A    1B    1C    1D    1E    1F */
          0x02, 0x02, 0x03, 0x02, 0x02, 0x02, 0x16, 0x07, 0x16, 0x02, 0x02, 0x03, 0x03, 0x03, 0x03, 0x03,
     
        /*20    21    22    23    24    25    26    27    28    29    2A    2B    2C    2D    2E    2F */
          0x07, 0x16, 0x05, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03,
    
        /*30    31    32    33    34    35    36    37    38    39    3A    3B    3C    3D    3E    3F */
          0x03, 0x06, 0x0B, 0x12, 0x16, 0x16, 0x16, 0x16, 0x03, 0x03, 0x03, 0x03, 0x03, 0x16, 0x16, 0x16
    };
    static InputPlugin g_plugin(WIIMOTE_INI_NAME, _trans("Wiimote"), "Wiimote");
    InputPlugin *GetPlugin() {
        return &g_plugin;
    }

    void Shutdown() {
        //dprintf("Shutdown ");
        std::vector<ControllerEmu*>::const_iterator
        i = g_plugin.controllers.begin(),
            e = g_plugin.controllers.end();
        for ( ; i!=e; ++i )
            delete *i;
        g_plugin.controllers.clear();

        // WiimoteReal is shutdown on app exit
        //WiimoteReal::Shutdown();

        g_controller_interface.Shutdown();
    }

    // if plugin isn't initialized, init and load config
    void Initialize(void* const hwnd) {

        //dprintf("Initialize ");
        // add 4 wiimotes
        for (unsigned int i = 0; i<4; ++i)
            g_plugin.controllers.push_back(new WiimoteEmu::Wiimote(i));

        g_controller_interface.SetHwnd(hwnd);
        g_controller_interface.Initialize();

        g_plugin.LoadConfig();

        WiiPC::Initialize();        

        // reload Wiimotes with our settings
        if (Movie::IsPlayingInput() || Movie::IsRecordingInput())
            Movie::ChangeWiiPads();
    }

    // __________________________________________________________________________________________________
    // Function: Wiimote_Output
    // Purpose:  An L2CAP packet is passed from the Core to the Wiimote,
    //           on the HID CONTROL channel.
    // input:    Da pakket.
    // output:   none
    //
    void ControlChannel(int _number, u16 _channelID, const void* _pData, u32 _Size) {
        #if 0
        dprintf("ControlChannel ");
        dprintf("(%d)[%d]:", _number, _channelID);
        dprintbuf(_pData, _Size);
        dprintf("\n");
        #endif               

        if (WIIMOTE_SRC_EMU & g_wiimote_sources[_number]){
            ((WiimoteEmu::Wiimote*)g_plugin.controllers[_number])->ControlChannel(_channelID, _pData, _Size);
        }

        if (WIIMOTE_SRC_REAL & g_wiimote_sources[_number] || ((WIIMOTE_SRC_WII_BALANCE_BOARD & g_wiimote_sources[_number]))) {			
			WiiPC::ControlChannel(_number, _channelID, _pData, _Size);
        }		
    }

    // __________________________________________________________________________________________________
    // Function: Wiimote_InterruptChannel
    // Purpose:  An L2CAP packet is passed from the Core to the Wiimote,
    //           on the HID INTERRUPT channel.
    // input:    Da pakket.
    // output:   none
    //
    void InterruptChannel(int _number, u16 _channelID, const void* _pData, u32 _Size) {
        if (WIIMOTE_SRC_EMU & g_wiimote_sources[_number])
            ((WiimoteEmu::Wiimote*)g_plugin.controllers[_number])->InterruptChannel(_channelID, _pData, _Size);
        else {			            
            WiiPC::InterruptChannel(_number, _channelID, _pData, _Size);
        }
    }

    // __________________________________________________________________________________________________
    // Function: Wiimote_Update
    // Purpose:  This function is called periodically by the Core.
    // input:    none
    // output:   none
    //
    void Update(int _number) {
        //PanicAlert( "Wiimote_Update" );
        // TODO: change this to a try_to_lock, and make it give empty input on failure
        std::lock_guard<std::recursive_mutex> lk(g_plugin.controls_lock);

        static int _last_number = 4;
        if (_number <= _last_number) {
            g_controller_interface.UpdateOutput();
            g_controller_interface.UpdateInput();
        }
        _last_number = _number;

        if (WIIMOTE_SRC_EMU & g_wiimote_sources[_number]) {
            ((WiimoteEmu::Wiimote*)g_plugin.controllers[_number])->Update();
        } else {
            //parywu//WiimoteReal::Update(_number);       			
            WiiPC::Update(_number);
        }	
    }

    // __________________________________________________________________________________________________
    // Function: PAD_GetAttachedPads
    // Purpose:  Get mask of attached pads (eg: controller 1 & 4 -> 0x9)
    // input:	 none
    // output:   number of pads
    //
    unsigned int GetAttached() {
        unsigned int attached = 0;
        for (unsigned int i=0; i<4; ++i)
            if (g_wiimote_sources[i])
                attached |= (1 << i);
        return attached;
    }

    // ___________________________________________________________________________
    // Function: DoState
    // Purpose:  Saves/load state
    // input/output: ptr
    // input: mode
    //
    void DoState(unsigned char **ptr, int mode) {
        // TODO:

        //PointerWrap p(ptr, mode);
        //for (unsigned int i=0; i<4; ++i)
        //	((WiimoteEmu::Wiimote*)g_plugin.controllers[i])->DoState(p);
    }

    // ___________________________________________________________________________
    // Function: EmuStateChange
    // Purpose: Notifies the plugin of a change in emulation state
    // input:    newState
    // output:   none
    //
    void EmuStateChange(EMUSTATE_CHANGE newState) {
        // TODO
        //parywu//WiimoteReal::StateChange(newState);
    }

    void LoadSettings() {
        std::string ini_filename = File::GetUserPath(D_CONFIG_IDX) + WIIMOTE_INI_NAME ".ini";

        IniFile inifile;
        inifile.Load(ini_filename);

        for (unsigned int i=0; i<MAX_WIIMOTES; ++i) {
            std::string secname("Wiimote");
            secname += (char)('1' + i);
            IniFile::Section& sec = *inifile.GetOrCreateSection(secname.c_str());

            sec.Get("Source", &g_wiimote_sources[i], i ? WIIMOTE_SRC_NONE : WIIMOTE_SRC_EMU);
        }
    }
    void Activiate(int index, bool connect){
        GetUsbPointer()->AccessWiiMote(index |0x100)->Activate(connect);
    }

    bool IsWiiRunning(void){
        return Core::IsRunning() && SConfig::GetInstance().m_LocalCoreStartupParameter.bWii;
    }

    bool IsCoreInited(void){
        return Core::GetState() != Core::CORE_UNINITIALIZED;
    }

    void CoreWiimoteInterruptChannel(int index, unsigned short conn_channel, unsigned char* pData){
        /* WiiPC to Core */
        uint8_t data[23];
        uint32_t n;
        uint8_t c = pData[0];

        if(c > 0x3F) {
            return;
        }

        n = wiimote_channel_sizes[c];

        memcpy(&data[1], pData, n);
        data[0] = 0xA1;

        Core::Callback_WiimoteInterruptChannel(index, conn_channel, data, n + 1);
    }

    void CoreWiimoteControlChannel(int index, unsigned short conn_channel, unsigned char* buffer, unsigned long size){
        Core::Callback_WiimoteInterruptChannel(index, conn_channel, buffer, size);
    }

    #if 0
    void CoreDisplayMessage(const char *message, int time_in_ms){
        //Core::DisplayMessage(message, time_in_ms);
        main_frame->StatusBarMessage(message);
    }
    #endif

}
