
#ifndef _WIIMOTE_H_
#define _WIIMOTE_H_

#include "../../InputCommon/Src/InputConfig.h"
#include <utility>

#define MAX_WIIMOTES	4

#define WIIMOTE_INI_NAME	"WiimoteNew"

enum
{
	WIIMOTE_SRC_NONE = 0,
	WIIMOTE_SRC_EMU = 1,
	WIIMOTE_SRC_REAL = 2,
	WIIMOTE_SRC_HYBRID = 3,	// emu + real
	WIIMOTE_SRC_WII_BALANCE_BOARD = 4,
};

typedef std::pair<u8*,u8> Report;

typedef void (*wm_sources_changed_callback_t)(void);

extern wm_sources_changed_callback_t wm_srcs_changed_callback;

struct wiimote_sources_t {
    struct item_t {
        int value;

        item_t():value(){}
        
        operator int(){
            return value;
        }

        int* operator&(){
            return &value;
        }

        void operator=(int _value){
            bool changed = (value != _value);
            value = _value;

            if(changed) {
                if(wm_srcs_changed_callback) {
                    wm_srcs_changed_callback();
                }
            }
        }
    };

    item_t items[MAX_WIIMOTES];

    wiimote_sources_t(){
        memset(items, 0, sizeof(items));
    }

    item_t& operator[](int index){
        return items[index];
    }
};


extern wiimote_sources_t g_wiimote_sources;


//extern unsigned int g_wiimote_sources[MAX_WIIMOTES];
//int     g_wiimote_sources_get(int index);
//void    g_wiimote_sources_set(int index, int value);

namespace Wiimote
{

void Shutdown();
void Initialize(void* const hwnd);

unsigned int GetAttached();
void DoState(unsigned char **ptr, int mode);
void EmuStateChange(EMUSTATE_CHANGE newState);
InputPlugin *GetPlugin();

void ControlChannel(int _number, u16 _channelID, const void* _pData, u32 _Size);
void InterruptChannel(int _number, u16 _channelID, const void* _pData, u32 _Size);
void Update(int _number);
void LoadSettings();


/* Interface to WiiPC */
void Activiate(int index, bool connect);
bool IsWiiRunning(void);

void CoreWiimoteInterruptChannel(int index, unsigned short conn_channel, unsigned char* buffer);
void CoreWiimoteControlChannel(int index, unsigned short conn_channel, unsigned char* buffer, unsigned long size);


bool IsCoreInited(void);
void CoreStartGame(void);
void CorePauseGame(void);
void CoreStopGame(void);

void CoreDisplayMessage(const char *message, int time_in_ms);

void CoreBringToFront(void);

}

#endif
