#include "BlobVolume.h"
#include "dconsole.h"

namespace DiscIO
{
typedef StaticTsM<IBlobReader*> 	StaticTsMIBlobReader;
typedef StaticTsM<TuVolume*>		StaticTsMVolume;

extern StaticTsMIBlobReader BlobFactory;
extern StaticTsMVolume      VolumeFactory;

static TsVwstring wsplit(const wstring& w, const wstring& gap){
	size_t p, q, n;
	TsVwstring v;

	p = q = 0;
	n = gap.length();

	do{
		p = w.find(gap, q);
		if(p > q){
			wstring x = w.substr(q, p-q);

			v->push_back(x);
			q = p + n;
		}
	}while(p > q);

	n = w.length();

	if(q < n){
		v->push_back(w.substr(q, n-q));
	}

	return v;
}

TuBlobVolume::TuBlobVolume(const wchar_t* _config) 
    : blob(), volume() 
{
    TsVwstring v = wsplit(_config, L"::");
    TsVwstring w = wsplit(v[0], L".");

    wstring blob_type = w[0];
    wstring volume_type = w[1];
    wstring blob_config = v[1];

    blob.reset(CreateBlobReaderByType(strwtos(blob_type).c_str(), blob_config.c_str()));
    volume.reset(CreateVolumeByType(strwtos(volume_type).c_str(), blob));

    volume->Identify();
}

wstring TuBlobVolume::GetConfig(void){
	wstring blob_type = strstow(blob->type);
	wstring volume_type = strstow(volume->type);
	return blob_type + L"." + volume_type + L"::" + blob->config;
}

TsBlobVolume    CreateBlobVolume(const wchar_t* config){
	return new TuBlobVolume(config);
}

TsBlobVolume    IdentifyBlobVolume(const wchar_t* path)	
{
	for(StaticTsMIBlobReader::Tu::iterator I = BlobFactory.ptr->begin(); I != BlobFactory.ptr->end(); ++I){
		for(StaticTsMVolume::Tu::iterator J = VolumeFactory.ptr->begin(); J != VolumeFactory.ptr->end(); ++J){		
			TsIBlobReader blob(I->second->create(path));
			TsVolume volume(J->second->create(blob));
			
			blob->type = I->second->type;
			volume->type = J->second->type;

			if(volume->Identify()){
				return new TuBlobVolume(blob, volume);
			}
		}
	}

	return 0;
}
} // namespace