/*
    Akar CC Ltd 2013
    Drive WBFS Blob
*/
#ifndef __DRIVEWBFSBLOB_H__
#define __DRIVEWBFSBLOB_H__

#include "Blob.h"
#include "iofile.h"
#include "TsAny.hpp"
#include <windows.h>

namespace DiscIO
{
const u64 wii_sector_size       = 0x8000;
const u64 wii_sector_count      = 143432 * 2;
const u64 wii_sector_log2       = 15;
const u64 wii_disc_header_size  = 256;

class DriveWbfsReader : public IBlobReader {
    DriveWbfsReader(const wchar_t* drive_letter, int disc_index /* from_zero */);

public:
    HANDLE  handle;
    u64     m_size;
    char    m_drive_letter;
    int     m_disc_index;

    u32     drv_sector_size;
    u32     drv_sector_count;

    TsBu8   drv_sector;
    u64     drv_sector_index;
    u32     drv_sector_offset;

    u64     hd_sector_size;
    u8      hd_sector_shift;
    u64     hd_sector_count;

    u64     wbfs_sector_size;
    u8      wbfs_sector_shift;
    u64     wbfs_sector_count;

    u64     m_blocks_per_disc;
    u64     m_disc_info_size;

    u64     m_freeblks_lba;
    u16     m_max_disc;

    TsBu8   disc_table;
    TsBu16  m_wlba_table;	
    bool    is_disc_loaded;

public:
	static DriveWbfsReader* Create(const wchar_t* filename);

	IBlobReader* create(const wchar_t* config){
		return Create(config);
	}

	DriveWbfsReader(IBlobReaderTypeInfo info): IBlobReader(info){}

	~DriveWbfsReader(){
		CloseHandle(handle);
	}


	bool Identify(void);

	static int GetDriveCapacity(const wchar_t* drive_letter, u32* sector_size, u32* sector_count, u64* size);
	u64 GetDataSize() const { return m_size;}
    u64 GetRawSize() const { return m_size;}

    bool RawSectorSeek(u64 sector_num);
    bool RawSectorRead(u64 sector_count, u8* out_ptr);

    bool Seek(u64 offset);
    bool Skip(u64 nbytes);

    bool RawRead(u64 offset, u64 nbytes, u8* out_ptr);
    bool RawRead(u64 nbytes, u8* out_ptr);

    u8   RawReadBigEndian8 (void);
    u16  RawReadBigEndian16(void);
    u32  RawReadBigEndian32(void);

    bool ReadHeader(void);
    bool LoadDisc(void);
    bool Read(u64 offset, u64 nbytes, u8* out_ptr);

    string DiscID(int index = -1 /* default = current */);
    u16 DiscCount(void);
    int FindDiscID(const string& id);
    int SetDiscIndexByID(const string& id);
};
} // namespace
#endif /* __DRIVEWBFSBLOB_H__ */
