#include "tsvolume.h"

namespace DiscIO
{
typedef StaticTsM<TuVolume*> StaticTsMVolume;
StaticTsMVolume VolumeFactory;

void VolumeRegister(const VolumeTypeInfo& info, TuVolume* meta){
	wstring _type(strstow(info.type));

	VolumeFactory.static_new();

	if(!VolumeFactory(_type)){
		VolumeFactory[_type] = meta;
		meta->type = string(info.type);
	}
}

TuVolume* CreateVolumeByType(const char* type, TsIBlobReader reader){
	wstring _type(strstow(type));

	if(VolumeFactory(_type)){
		TuVolume* p = VolumeFactory[_type];
		TuVolume* q = p->create(reader);
		q->type = p->type;
		return q;
	}

	return 0;
}

static inline unsigned short byteswap_ushort(unsigned short value)
{
	return (value >> CHAR_BIT) | (value << CHAR_BIT);
}

static inline unsigned long byteswap_ulong(unsigned long value)
{
	return _byteswap_ushort((unsigned short)value) << (2 * CHAR_BIT) |
		   _byteswap_ushort((unsigned short)(value >> (2 * CHAR_BIT)));
}

bool TuVolume::RawRead(u64 offset, u64 size, u8* buffer){
    if(blob.get()) {
        return blob->Read(offset, size, buffer);
    }
    return false;
}

u16  TuVolume::RawReadBigEndian16(u64 offset, u16 def_value){
    u16 v;
    return RawRead(offset, sizeof(v), (u8*)&v) ? byteswap_ushort(v) : def_value;
}

u32  TuVolume::RawReadBigEndian32(u64 offset, u32 def_value){
    u32 v;
    return RawRead(offset, sizeof(v), (u8*)&v) ? byteswap_ulong(v) : def_value;
}
}

